@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# Copyright (C) 2000-2003 the xine project
+# Copyright (C) 2000-2006 the xine project
 #
 # This file is part of xine, a unix video player.
 # 
@@ -18,8 +18,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 #
-# $Id: autogen.sh,v 1.12 2006-02-21 19:06:09 dsalt Exp $
-#
 # Maintained by Stephen Torri <storri@users.sourceforge.net>
 #
 # run this to generate all the initial makefiles, etc.
@@ -27,9 +25,10 @@
 PROG=gxine
 
 # Minimum value required to build
-WANT_AUTOMAKE_1_9=1 export WANT_AUTOMAKE_1_9
-AUTOMAKE_MIN=1.9.0
-AUTOCONF_MIN=2.53
+WANT_AUTOMAKE_1_8=1 export WANT_AUTOMAKE_1_8
+WANT_AUTOMAKE=1.8 export WANT_AUTOMAKE
+AUTOMAKE_MIN=1.8.0
+AUTOCONF_MIN=2.59
 LIBTOOL_MIN=1.4.0
 
 # Check how echo works in this /bin/sh
@@ -200,7 +199,7 @@ run_automake () {
 #-------------------
 detect_aclocal() {
 
-  # if no automake, don't bother testing for aclocal
+  # if no automake, don't bother testing for aclocal;
   set -- `type aclocal 2>/dev/null`
   RETVAL=$?
   NUM_RESULT=$#
@@ -229,9 +228,14 @@ run_aclocal () {
   fi
   
   echo $_echo_n " + Running aclocal: $_echo_c"
-  aclocalinclude=`if [ ! -z "$XINE_CONFIG" ]; then $XINE_CONFIG --acflags; else xine-config --acflags; fi`
+  
+  aclocalinclude=`if [ ! -z "$XINE_CONFIG" ]; then
+		    $XINE_CONFIG --acflags
+		  else
+		    pkg-config --atleast-version 1.1.90 libxine && pkg-config --variable=acflags libxine || xine-config --acflags
+		  fi`
 
-  aclocal -I m4 $aclocalinclude
+  aclocal -I m4 -I m4/gettext $aclocalinclude
   echo "done." 
 }
 
