@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2007 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -17,8 +17,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
  *
- * $Id: plugin.c,v 1.33 2006-02-21 19:06:11 dsalt Exp $
- *
  * xine plugin for mozilla
  *
  */
@@ -156,6 +154,8 @@ char *NPP_GetMIMEDescription(void) {
          "audio/x-mpeg3: mp3: MPEG audio;"
          "audio/mpeg: mpa,abs,mpega: MPEG audio;"
          "audio/x-mpeg: mpa,abs,mpega: MPEG audio;"
+         "audio/mpegurl: m3u: MPEG URL;"
+         "audio/x-mpegurl: m3u: MPEG URL;"
          "video/quicktime: mov,qt: Quicktime animation;"
          "video/x-quicktime: mov,qt: Quicktime animation;"
          "video/msvideo: avi: AVI animation;"
