@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2005 the xine-project
+ * Copyright (C) 2003-2007 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,8 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: client.c,v 1.11 2006/03/27 22:48:57 dsalt Exp $
- *
  * gxine Javascript client
  */
 
@@ -35,7 +33,8 @@
 #include <fcntl.h>
 #include <signal.h>
 
-#define SOCKET_FILENAME "%s/.gxine/socket"
+#include <glib.h>
+
 #define BUF_LEN 1024
 
 static volatile int done = 0;
@@ -69,7 +68,11 @@ int main (int argc, char **argv)
   console_output_init ();
 
   /* server filename */
-  snprintf (filename, sizeof (filename), SOCKET_FILENAME, getenv ("HOME"));
+  const char *cfgdir = g_get_user_config_dir ();
+  int home_is_not_cfgdir = !!strcmp (g_get_home_dir (), cfgdir);
+  char *path = g_build_filename (cfgdir, ".gxine" + home_is_not_cfgdir, "socket", NULL);
+  snprintf (filename, sizeof (filename), "%s", path);
+  g_free (path);
   if (strlen (filename) >= sizeof (cli_adr.u.sun_path))
   {
     fputs (_("socket: name too long - cannot connect\n"), stderr);
