@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2005 the xine-project
+ * Copyright (C) 2002-2006 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,8 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: desktop_integration.c,v 1.21 2006/03/27 22:11:24 dsalt Exp $
- *
  * helper functions to register gxine with kde and gnome
  * (and possibly cde in the future)
  *
@@ -43,6 +41,9 @@
 #include "desktop_integration.h"
 #include "utils.h"
 
+#define LIBGNOMEVFS "libgnomevfs-2.so.0"
+#define LIBGNOMEUI  "libgnomeui-2.so.0"
+
 typedef enum { GNOME_VFS_OK } GnomeVFSResult; /* incomplete */
 
 /*
@@ -56,10 +57,8 @@ struct gnome_vfs_functions
   void (*application_registry_set_value)	  (const char *app_id, const char *key, const char *value);
   void (*application_registry_set_bool_value)	  (const char *app_id, const char *key, int value);
   void (*application_registry_remove_mime_type)	  (const char *app_id, const char *mime_type);
-#ifndef GNOME_VFS_DISABLE_DEPRECATED
   void (*mime_freeze) (void);
   void (*mime_thaw)   (void);
-#endif
 };
 
 /*
@@ -84,16 +83,19 @@ struct mime_info
 #define gnome_vfs_application_registry_set_value gnome_vfs->application_registry_set_value
 #define gnome_vfs_application_registry_set_bool_value gnome_vfs->application_registry_set_bool_value
 
-#define gnome_vfs_mime_thaw gnome_vfs->mime_thaw
-#define gnome_vfs_mime_freeze gnome_vfs->mime_freeze
+#define gnome_vfs_mime_freeze() do { if (gnome_vfs->mime_freeze) gnome_vfs->mime_freeze (); } while (0)
+#define gnome_vfs_mime_thaw()   do { if (gnome_vfs->mime_thaw)   gnome_vfs->mime_thaw ();   } while (0)
 
 static void * gnome_vfs_handle = NULL;
 static int gnome_vfs_available = -1;
 static struct gnome_vfs_functions * gnome_vfs = NULL;
 
+static void * gnome_ui_handle = NULL;
+
 
 /*
- * load the gnome-vfs library and initialize the function pointers
+ * load the GnomeVFS and GnomeUI library
+ * If using the integration wizard, also initialize the function pointers
  */
 
 typedef void (*func_t) ();
@@ -116,16 +118,18 @@ static func_t get_func (const char *name)
 			 get_func ("gnome_vfs_" #VAR))) \
     return gnome_vfs_available = 0
 
-gboolean di_gnome_vfs_init (void)
+#define GET_FUNC_S_I(VAR) \
+  gnome_vfs->VAR = (typeof (gnome_vfs->VAR)) get_func ("gnome_vfs_" #VAR)
+
+gboolean gxine_vfs_init (void)
 {
-  void (*gnome_vfs_init_func) (void) = NULL;
-  void (*gnome_vfs_mime_get_value) (const char *, const char *) = NULL;
+  void (*init_func) (void) = NULL;
 
   if (gnome_vfs_available != -1)
     return gnome_vfs_available;
 
   /* load the gnomevfs library */
-  gnome_vfs_handle = dlopen ("libgnomevfs-2.so.0", RTLD_LAZY);
+  gnome_vfs_handle = dlopen (LIBGNOMEVFS, RTLD_LAZY);
   if (!gnome_vfs_handle)
   {
     fprintf (stderr, "%s.\n", dlerror());
@@ -135,20 +139,38 @@ gboolean di_gnome_vfs_init (void)
   gnome_vfs = (struct gnome_vfs_functions *) calloc (1, sizeof (struct gnome_vfs_functions));
 
   /* this call causes gnome-vfs to become initialized */
-  GET_FUNC (gnome_vfs_init_func, "gnome_vfs_init");
-  gnome_vfs_init_func ();
+  GET_FUNC (init_func, "gnome_vfs_init");
+  init_func ();
+
+  /* load the gnomeui library (for the authentication bits) */
+  gnome_ui_handle = dlopen (LIBGNOMEUI, RTLD_LAZY);
+  if (gnome_ui_handle)
+  {
+    /* this call causes gnome-vfs to become initialized */
+    init_func = dlsym (gnome_ui_handle, "gnome_authentication_manager_init");
+    if (init_func)
+      init_func ();
+    else
+      fprintf (stderr, "%s.\n", dlerror());
+  }
+  else
+    fprintf (stderr, "%s.\n", dlerror());
+
+#ifdef USE_INTEGRATION_WIZARD
+  void (*gnome_vfs_mime_get_value) (const char *, const char *) = NULL;
 
   GET_FUNC (gnome_vfs_mime_get_value, "gnome_vfs_mime_get_value");
   gnome_vfs_mime_get_value ("text/plain", "description");
 
   /* extract needed symbols from the library */
-  GET_FUNC_S (mime_freeze);
-  GET_FUNC_S (mime_thaw);
+  GET_FUNC_S_I (mime_freeze);
+  GET_FUNC_S_I (mime_thaw);
   GET_FUNC_S (application_registry_remove_application);
   GET_FUNC_S (application_registry_add_mime_type);
   GET_FUNC_S (application_registry_set_value);
   GET_FUNC_S (application_registry_set_bool_value);
   GET_FUNC_S (application_registry_remove_mime_type);
+#endif
 
   return gnome_vfs_available = 1;
 }
@@ -162,7 +184,7 @@ gboolean di_gnome_vfs_init (void)
   if ((VAR = (typeof (VAR)) get_func ((NAME))) == NULL) \
     return
 
-static void gnome_vfs_shutdown (void)
+static void di_gnome_vfs_flush (void)
 {
   /* release the gnomevfs library */
   if (gnome_vfs_handle)
@@ -173,7 +195,7 @@ static void gnome_vfs_shutdown (void)
     GET_FUNC (gnome_vfs_application_registry_sync, "gnome_vfs_application_registry_sync");
 
     gnome_vfs_application_registry_sync ();
-    dlclose (gnome_vfs_handle);
+    /* dlclose (gnome_vfs_handle); */
   }
 
   free (gnome_vfs);
@@ -270,7 +292,7 @@ void di_register_gnome (void)
 
   section = "GNOME";
 
-  if (!di_gnome_vfs_init())
+  if (!gxine_vfs_init())
   {
     di_report (_("GNOME initialisation failed"));
     return;
@@ -309,7 +331,7 @@ void di_register_gnome (void)
   /* force the user.mime and user.keys to be written  */
   gnome_vfs_mime_thaw();
 
-  gnome_vfs_shutdown();
+  di_gnome_vfs_flush();
 }
 
 void di_register_kde (void)
