@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2006 the xine-project
+ * Copyright (C) 2003-2007 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,8 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: engine.c,v 1.36 2006/04/04 22:39:24 dsalt Exp $
- *
  * init xine engine, set up script engine
  */
 
@@ -30,6 +28,7 @@
 #include <pthread.h>
 #include <stdarg.h>
 #include <errno.h>
+#include <unistd.h>
 
 #include <glib/gthread.h>
 
@@ -56,10 +55,6 @@ static gboolean modified = FALSE;
 GAsyncQueue *js_queue;
 static gboolean queue_available = FALSE;
 
-#if GXINE_GTK_OLDER_THAN(2,8,0)
-WEAK (void, gtk_about_dialog_set_wrap_license);
-#endif
-
 static JSBool
 show_about (JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
@@ -84,8 +79,8 @@ show_about (JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
     GtkBox *vbox;
     /* translators' names go here */
     const gchar *translator = _("TRANSLATOR");
-    gchar *licence = GTK_HAVE (gtk_about_dialog_set_wrap_license, 2, 8, 0)
-      ? _("This program is free software; you can redistribute it and/or "
+    gchar *licence = 
+	_("This program is free software; you can redistribute it and/or "
 	  "modify it under the terms of the GNU General Public Licence as "
 	  "published by the Free Software Foundation; either version 2 of the "
 	  "Licence, or (at your option) any later version.\n"
@@ -98,24 +93,6 @@ show_about (JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 	  "You should have received a copy of the GNU General Public Licence "
 	  "along with this program; if not, write to the Free Software "
 	  "Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, "
-	  "MA  02110-1301  USA")
-      /* Translators: rewrap as required */
-      : _("This program is free software; you can\n"
-	  "redistribute it and/or modify it under the terms of\n"
-	  "the GNU General Public Licence as published by\n"
-	  "the Free Software Foundation; either version 2 of\n"
-	  "the Licence, or (at your option) any later version.\n"
-	  "\n"
-	  "This program is distributed in the hope that it will\n"
-	  "be useful, but WITHOUT ANY WARRANTY; without\n"
-	  "even the implied warranty of MERCHANTABILITY\n"
-	  "or FITNESS FOR A PARTICULAR PURPOSE. See the\n"
-	  "GNU General Public Licence for more details.\n"
-	  "\n"
-	  "You should have received a copy of the GNU\n"
-	  "General Public Licence along with this program;\n"
-	  "if not, write to the Free Software Foundation, Inc.,\n"
-	  "51 Franklin St, Fifth Floor, Boston,\n"
 	  "MA  02110-1301  USA");
 
     about = gtk_about_dialog_new ();
@@ -125,17 +102,21 @@ show_about (JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 		  "copyright", get_copyright_notice (),
 		  "comments", _("A GTK media player front end for xine-lib"),
 		  "license", licence,
+		  "wrap-license", TRUE,
 		  "website", "http://xinehq.de/",
 		  "authors", authors,
 		  "documenters", documenters,
 		  "translator-credits",
 		  strcmp (translator, "TRANSLATOR") ? translator : NULL,
 		  "logo-icon-name", GXINE_LOGO, NULL);
-    if (GTK_HAVE (gtk_about_dialog_set_wrap_license, 2, 8, 0))
-      gtk_about_dialog_set_wrap_license ((GtkAboutDialog *) about, TRUE);
 
-    label = gtk_label_new ("<small><i>" VENDOR_PKG_VERSION CSET_ID "</i></small>");
-    gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+    char *version_label =
+      g_strdup_printf (_("<small><i>%s\nUsing xine-lib %s, built with %s</i></small>"),
+		       VENDOR_PKG_VERSION CSET_ID,
+		       xine_get_version_string (), XINE_VERSION);
+    label = ui_label_new_with_markup (version_label);
+    gtk_label_set_justify ((GtkLabel *) label, GTK_JUSTIFY_CENTER);
+    free (version_label);
     vbox = GTK_BOX (GTK_DIALOG (about)->vbox);
     gtk_box_pack_end (vbox, label, FALSE, FALSE, 2);
     g_object_connect (about,
@@ -155,6 +136,7 @@ print_cb (void *user_data, const char *str, ...)
   if (verbosity)
   {
     va_list ap;
+    /* JS diagnostic output */
     g_print (_("engine: < "));
     va_start (ap, str);
     vprintf (str, ap);
@@ -201,6 +183,7 @@ int engine_exec_obj (const char *cmd, se_o_t *obj, se_print_cb_t cb,
 
   if ((str = se_result_str (gse)))
   {
+    /* JS output. Could end up in a terminal or the JS console. */
     str = g_strdup_printf (_("result: %s"), str);
     cb (cb_data, "%s", str);
     free (str);
@@ -340,7 +323,7 @@ load_user_startup_script (void)
   char *text = read_entire_file (file, NULL);
   if (!text)
   {
-    text = strdup ("");
+    text = strdup ("\n");
     if (errno != ENOENT)
       display_error (FROM_GXINE, _("Couldn't load startup script"), "%s: %s",
 		     file, strerror (errno));
@@ -443,6 +426,7 @@ void engine_init (void)
   xine = xine_new ();
 
   cfgfilename = get_config_filename (FILE_CONFIG);
+  have_config = !access (cfgfilename, R_OK); /* upgrade or install? */
   xine_config_load (xine, cfgfilename);
   free (cfgfilename);
   xine_engine_set_param (xine, XINE_ENGINE_PARAM_VERBOSITY, verbosity);
@@ -505,6 +489,7 @@ engine_startup_cb (gpointer data)
   g_signal_connect (G_OBJECT (app), "client-event",
 		    G_CALLBACK (js_queue_cb), NULL);
   js_queue_cb (NULL, NULL, NULL);
+  gxine_init_done ();
   return FALSE;
 }
 
@@ -538,7 +523,7 @@ engine_startup_script (void)
   se_startup_cmds = load_user_startup_script ();
   se_eval (gse, se_startup_cmds, NULL, NULL, NULL, _("Startup script"));
 
-  gtk_init_add ((GtkFunction) engine_startup_cb, NULL);
+  gxine_init_add ((GtkFunction) engine_startup_cb, NULL);
 }
 
 void
