@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2004 the xine-project
+ * Copyright (C) 2001-2006 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,13 +16,11 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: globals.h,v 1.39 2006-03-03 02:01:16 dsalt Exp $
- *
  * global variables for lazy programmers... :>
  */
 
-#ifndef HAVE_GLOBALS_H
-#define HAVE_GLOBALS_H
+#ifndef GXINE_GLOBALS_H
+#define GXINE_GLOBALS_H
 
 /*
 #define EXP_STUFF
@@ -43,6 +41,17 @@
 
 #include "defs.h"
 
+typedef enum {
+  CONFIG_VERSION_BASE,
+  CONFIG_VERSION_0_6_0,
+  CONFIG_VERSION_CURRENT = CONFIG_VERSION_0_6_0
+} config_version_t;
+
+extern gboolean have_config;
+extern int config_version;
+#define CONFIG_VERSION_ATLEAST(EPOCH, MAJOR, MINOR) \
+  (!have_config || config_version >= CONFIG_VERSION_##EPOCH##_##MAJOR##_##MINOR)
+
 extern GtkWidget      *app;
 extern GtkWidget      *gtv; /* GtkVideo widget */
 extern GtkWidget      *menubar, *popup_menu;
@@ -57,7 +66,7 @@ extern GdkPixbuf      *gxine_logo;
 typedef struct {
   GtkToggleAction *fullscreen, *resize_factor, *auto_resize, *auto_rescale,
 		  *deinterlace, *vo_postproc, *ao_postproc, *fs_toolbar,
-		  *wm_toolbar, *wm_toolbar_snap, *wm_unblank;
+		  *wm_toolbar, *wm_toolbar_snap, *wm_unblank, *fs_sticky;
   GSList *aspect, *vis, *fs_toolbar_pos, *video_size, *subtitles;
   GtkAction *keypad;
 } action_items_t;
@@ -75,6 +84,8 @@ extern xine_video_port_t *video_port;
 
 extern GAsyncQueue    *js_queue;
 
+extern gboolean        fs_always_sticky;
+
 #define MAX_MRL_LEN 1024
 
 #define CONFIGDIR		"."PACKAGE
