@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2005 the xine project
+ * Copyright (C) 2004-2006 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,13 +15,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: gtkflipbutton.h,v 1.4 2006-02-21 19:06:11 dsalt Exp $
- *
  * a GTK toggle-button-like widget
  */
 
-#ifndef HAVE_GTK_FLIP_BUTTON_H
-#define HAVE_GTK_FLIP_BUTTON_H
+#ifndef GXINE_GTK_FLIP_BUTTON_H
+#define GXINE_GTK_FLIP_BUTTON_H
 
 #include <gtk/gtktogglebutton.h>
 
@@ -62,9 +60,11 @@ extern "C"
   GtkWidget *gtk_flip_button_new (GtkWidget *, GtkWidget *);
   GtkWidget *gtk_flip_button_new_from_stock (const char *, const char *,
   					     GtkIconSize);
+  GtkWidget *gtk_flip_button_new_from_icon_names (const char *, const char *,
+						  GtkIconSize);
 
 #ifdef __cplusplus
 }
 #endif				/* __cplusplus */
 
-#endif				/* HAVE_GTK_FLIP_BUTTON_H */
+#endif				/* GXINE_GTK_FLIP_BUTTON_H */
