@@ -753,7 +753,7 @@ static void frame_output_cb (void *gv_gen,
   {
     if (priv->frame_cb_id)
       g_source_remove (priv->frame_cb_id);
-    priv->frame_cb_id = g_idle_add (priv->frame_cb, priv->frame_cb_data);
+    priv->frame_cb_id = g_idle_add ((GSourceFunc) priv->frame_cb, priv->frame_cb_data);
   }
 }
 
