@@ -1,23 +1,22 @@
 /*
- * Copyright (C) 2002-2005 the xine project
- * 
+ * Copyright (C) 2002-2006 the xine project
+ *
  * This file is part of xine, a free video player.
- * 
+ *
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * xine is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
  *
- * $Id: http.c,v 1.28 2006/04/08 14:44:44 dsalt Exp $
  */
 
 #include "globals.h"
@@ -34,23 +33,26 @@
 #include <unistd.h>
 #include <ctype.h>
 #include <errno.h>
+#include <poll.h>
+#include <time.h>
+
+#include <netinet/in.h>
 
 #include "http.h"
 #include "utils.h"
+#include "watchdog.h"
 
 /*
 #define LOG
 */
 
 #define BUFSIZE                 1024
+#define BUFSIZE_STR		"1023"
 #define DEFAULT_HTTP_PORT         80
 
 typedef struct http_s http_t;
 
 struct http_s {
-
-  int              fh;
-
   off_t            curpos;
   off_t            contentlength;
 
@@ -62,18 +64,22 @@ struct http_s {
 
   char             auth[BUFSIZE];
   char             proxyauth[BUFSIZE];
-  
+
   char            *user;
   char            *password;
   char            *host;
   int              port;
+
+  int              fh;
+
   char            *filename;
-  
+
   char            *proxyuser;
   char            *proxypassword;
   char            *proxyhost;
   int              proxyport;
 
+  int		   redir_count;
 };
 
 static int http_host_connect_attempt (struct in_addr ia, int port)
@@ -82,7 +88,7 @@ static int http_host_connect_attempt (struct in_addr ia, int port)
     struct sockaddr_in i;
     struct sockaddr s;
   } sin;
-	
+
   int s = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
   if (s == -1)
   {
@@ -91,20 +97,57 @@ static int http_host_connect_attempt (struct in_addr ia, int port)
     return -1;
   }
 
-  sin.i.sin_family = AF_INET;	
+  sin.i.sin_family = AF_INET;
   sin.i.sin_addr   = ia;
   sin.i.sin_port   = htons(port);
-	
-  if (connect (s, &sin.s, sizeof (sin))==-1 
-      && errno != EINPROGRESS)
+
+  int blocking = fcntl (s, F_SETFL, fcntl (s, F_GETFL) | O_NONBLOCK) == -1;
+  if (blocking)
+    watchdog_off ();
+
+  if (!connect (s, &sin.s, sizeof (sin)))
   {
-    display_error (FROM_GXINE, _("HTTP error"),
-		   _("Cannot connect to host: %s"), strerror (errno));
-    close (s);
-    return -1;
-  }	
-	
-  return s;
+    if (blocking)
+      watchdog_on ();
+    return s;
+  }
+
+  if (errno != EINPROGRESS)
+    goto fail;
+
+  /* if we get here, we have a non-blocking socket not yet connected */
+
+  struct pollfd fd = { s, POLLIN | POLLPRI | POLLOUT | POLLERR | POLLHUP };
+
+  time_t start = time (NULL);
+  do
+  {
+    switch (poll (&fd, 1, 1000))
+    {
+    case -1:
+      goto fail;
+    case 0:
+      do_pending_events ();
+      continue;
+    default:
+      if (fd.revents & POLLERR)
+	goto fail;
+      if (fd.revents & POLLHUP)
+      {
+	errno = ETIMEDOUT; /* or something */
+	goto fail;
+      }
+      return s;
+    }
+  } while (time (NULL) - start < 30);
+
+  fail:
+  display_error (FROM_GXINE, _("HTTP error"),
+		 _("Cannot connect to host: %s"), strerror (errno));
+  close (s);
+  if (blocking)
+    watchdog_on ();
+  return -1;
 }
 
 static int http_host_connect (const char *host, int port)
@@ -116,7 +159,7 @@ static int http_host_connect (const char *host, int port)
 		   _("Unable to resolve ‘%s’"), host);
     return -1;
   }
-	
+
   int i, s;
   for (i = 0, s = -1; s == -1 && h->h_addr_list[i]; ++i)
   {
@@ -136,104 +179,106 @@ static int http_parse_url (char *urlbuf, char **user, char **password,
 
   if (user != NULL)
     *user = NULL;
-  
+
   if (password != NULL)
     *password = NULL;
-  
+
   if (host != NULL)
     *host = NULL;
-  
+
   if (filename != NULL)
     *filename = NULL;
-  
+
   if (port != NULL)
     *port = 0;
-  
+
   start = strstr(urlbuf, "://");
   if (start != NULL)
     start += 3;
   else
     start = urlbuf;
-  
+
   at = strchr(start, '@');
   slash = strchr(start, '/');
-  
+
   if (at != NULL && slash != NULL && at > slash)
     at = NULL;
-  
+
   if (at != NULL)
   {
     authcolon = strchr(start, ':');
     if(authcolon != NULL && authcolon > at)
       authcolon = NULL;
-    
+
     portcolon = strchr(at, ':');
   }
   else
     portcolon = strchr(start, ':');
-  
+
   if (portcolon != NULL && slash != NULL && portcolon > slash)
     portcolon = NULL;
-  
+
   if (at != NULL)
   {
     *at = '\0';
-    
+
     if (user != NULL)
       *user = start;
-    
+
     if (authcolon != NULL)
     {
       *authcolon = '\0';
-      
+
       if (password != NULL)
       	*password = authcolon + 1;
     }
-    
+
     if (host != NULL)
       *host = at + 1;
   } else
     if (host != NULL)
       *host = start;
-  
+
   if (slash != NULL)
   {
     *slash = '\0';
-    
+
     if (filename != NULL)
       *filename = slash + 1;
   } else if (filename != NULL)
     *filename = urlbuf + strlen(urlbuf);
-  
+
   if (portcolon != NULL)
   {
     *portcolon = '\0';
-    
+
     if (port != NULL)
       *port = atoi(portcolon + 1);
   }
-  
+
   return 0;
 }
 
 static int http_basicauth (const char *user, const char *password,
-			   char* dest, int len) {
+			   char* dest, int len)
+{
   static const char enctable[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
   char        *tmp;
   char        *sptr;
   char        *dptr;
   int          count;
-  
+
   count = strlen (user) + 1;
   if (password != NULL)
     count += strlen (password);
-  
-  if (len < ((count + 2) / 3 ) * 4 + 1)
+
+  if (len < ((count + 2) / 3) * 4 + 1)
     return -1;
-  
+
   sptr = tmp = g_strconcat (user, ":", password, NULL);
   dptr = dest;
-  while (count >= 3) {
+  while (count >= 3)
+  {
     *dptr++ = enctable[(sptr[0] & 0xFC) >> 2];
     *dptr++ = enctable[((sptr[0] & 0x3) << 4) | ((sptr[1] & 0xF0) >> 4)];
     *dptr++ = enctable[((sptr[1] & 0x0F) << 2) | ((sptr[2] & 0xC0) >> 6)];
@@ -241,10 +286,11 @@ static int http_basicauth (const char *user, const char *password,
     count -= 3;
     sptr += 3;
   }
-  
-  if (count > 0) {
+
+  if (count > 0)
+  {
     *dptr++ = enctable[(sptr[0] & 0xFC) >> 2];
-    
+
     if (count > 1)
     {
       *dptr++ = enctable[((sptr[0] & 0x3) << 4) | ((sptr[1] & 0xF0) >> 4)];
@@ -255,12 +301,12 @@ static int http_basicauth (const char *user, const char *password,
       *dptr++ = enctable[(sptr[0] & 0x3) << 4];
       *dptr++ = '=';
     }
-    
+
     *dptr++ = '=';
   }
-  
+
   dptr[0] = '\0';
-  
+
   free (tmp);
   return 0;
 }
@@ -279,61 +325,72 @@ report_error (const char *url, const char *fmt, ...)
   free (msg);
 }
 
-static http_t *http_open (const char *mrl) {
-
-  http_t   *this;
+static http_t *http_open (const char *mrl)
+{
   char     *proxy, *request;
-  int       done,len,linenum;
+  int       done, len, linenum;
   char      mime_type[BUFSIZE];
+  char      location[BUFSIZE];
+  int       redirect = 0;
 
-  this = malloc (sizeof (http_t));
+  http_t *this = malloc (sizeof (http_t));
 
-  strncpy (this->mrlbuf, mrl, BUFSIZE);
+  this->redir_count = 0;
+  snprintf (this->mrlbuf, sizeof (this->mrlbuf), "%s", mrl);
 
   this->proxybuf[0] = '\0';
   proxy = getenv("http_proxy");
-  
-  if (proxy != NULL)  {
-    strncpy (this->proxybuf, proxy, BUFSIZE);
-    
+
+  retry:
+
+  strcpy (location, this->mrlbuf);
+
+  if (proxy != NULL)
+  {
+    snprintf (this->proxybuf, sizeof (this->proxybuf), "%s", proxy);
+
     if (http_parse_url (this->proxybuf, &this->proxyuser,
-			&this->proxypassword, &this->proxyhost, 
-			&this->proxyport, NULL)) {
+			&this->proxypassword, &this->proxyhost,
+			&this->proxyport, NULL))
+    {
       /* malformed URL */
-      report_error (mrl, _("URL parse error"));
+      report_error (location, _("URL parse error"));
       free (this);
       return NULL;
     }
-    
+
     if (this->proxyport == 0)
       this->proxyport = DEFAULT_HTTP_PORT;
-    
-    if (this->proxyuser != NULL)
-      if (http_basicauth (this->proxyuser, this->proxypassword,
-			  this->proxyauth, BUFSIZE)) {
-	report_error (mrl, _("proxy authentication error"));
-	free (this);
-	return NULL;
-      }
+
+    if (this->proxyuser != NULL &&
+        http_basicauth (this->proxyuser, this->proxypassword,
+			this->proxyauth, BUFSIZE))
+    {
+      report_error (location, _("proxy authentication error"));
+      free (this);
+      return NULL;
+    }
   }
-  
+
   if (http_parse_url (this->mrlbuf, &this->user, &this->password,
-		      &this->host, &this->port, &this->filename)) {
+		      &this->host, &this->port, &this->filename))
+  {
     free (this);
     /* malformed URL */
-    report_error (mrl, _("URL parse error"));
+    report_error (location, _("URL parse error"));
     return NULL;
   }
 
   if (this->port == 0)
     this->port = DEFAULT_HTTP_PORT;
 
-  if (this->user != NULL)
-    if (http_basicauth (this->user, this->password, this->auth, BUFSIZE)) {
-      report_error (mrl, _("proxy authentication error"));
-      free (this);
-      return NULL;
-    }
+  if (this->user != NULL &&
+      http_basicauth (this->user, this->password, this->auth, BUFSIZE))
+  {
+    report_error (location, _("proxy authentication error"));
+    free (this);
+    return NULL;
+  }
 
   if (proxy != NULL)
     logprintf ("http: opening /%s on host %s via proxy %s",
@@ -349,8 +406,9 @@ static http_t *http_open (const char *mrl) {
 
   this->curpos = 0;
 
-  if (this->fh == -1) { /* couldn't connect */
-    report_error (mrl, _("host connect error"));
+  if (this->fh == -1)
+  { /* couldn't connect */
+    report_error (location, _("host connect error"));
     free (this);
     return NULL;
   }
@@ -385,9 +443,13 @@ static http_t *http_open (const char *mrl) {
 	      "%sUser-Agent: xine/%s\015\012Accept: */*\015\012\015\012",
 	      request, VERSION);
 
-  if (write (this->fh, request, strlen(request)) != strlen(request)) {
+  len = strlen (request);
+  if (write (this->fh, request, len) != len)
+  {
     free (request);
-    report_error (mrl, _("couldn't send request"));
+    report_error (location, _("couldn't send request"));
+abort:
+    close (this->fh);
     free (this);
     return NULL;
   }
@@ -396,30 +458,31 @@ static http_t *http_open (const char *mrl) {
   free (request);
 
   /* read and parse reply */
-  done = 0; len = 0; linenum = 0;
+  done = len = linenum = 0;
   this->contentlength = 0;
 
-  while (!done) {
-
-    if (read (this->fh, &this->buf[len], 1) <=0) {
-      
-      switch (errno) {
+  while (!done)
+  {
+    if (read (this->fh, &this->buf[len], 1) <=0)
+    {
+      switch (errno)
+      {
       case EAGAIN:
 	logprintf ("http: EAGAIN\n");
 	continue;
       default:
-	report_error (mrl, _("read error: %s"), strerror (errno));
-	free (this);
-	return NULL;
+	report_error (location, _("read error: %s"), strerror (errno));
+	goto abort;
       }
     }
 
-    if (this->buf[len] == '\012' || len >= (BUFSIZE - 2)) {
-
+    if (this->buf[len] == '\012' || len >= (BUFSIZE - 2))
+    {
       this->buf[len] = '\0';
       len--;
-      
-      if (len >= 0 && this->buf[len] == '\015') {
+
+      if (len >= 0 && this->buf[len] == '\015')
+      {
 	this->buf[len] = '\0';
 	len--;
       }
@@ -428,49 +491,74 @@ static http_t *http_open (const char *mrl) {
 
       logprintf ("input_http: answer: >%s<\n", this->buf);
 
-      if (linenum == 1) {
+      if (linenum == 1)
+      {
         int httpver, httpsub, httpcode;
 	char httpstatus[BUFSIZE];
-	
+
 	if (sscanf(this->buf, "HTTP/%d.%d %d %[^\015\012]", &httpver, &httpsub,
-		   &httpcode, httpstatus) != 4)	{
-	  report_error (mrl, _("invalid HTTP response"));
-	  free (this);
-	  return NULL;
+		   &httpcode, httpstatus) != 4)
+	{
+	  report_error (location, _("invalid HTTP response"));
+	  goto abort;
 	}
-	
-	if (httpcode >= 300 && httpcode < 400) {
-	  /* the server says that the resource is elsewhere */
-	  report_error (mrl, _("HTTP redirection not implemented"));
-	  free (this);
-	  return NULL;
+
+	switch (httpcode)
+	{
+	/* The following codes have cacheable Location headers,
+	 * but we don't yet handle updating the MRLs.
+	 */
+	case 300: /* Multiple Choices */
+	case 301: /* Moved Permanently */
+	/* The following codes have uncacheable Location headers. */
+	case 302: /* Found */
+	case 303: /* See Other */
+	case 307: /* Temporary Redirect */
+	  if (++this->redir_count >= 10)
+	  {
+	    report_error (mrl, _("Too many HTTP redirections; aborted"));
+	    goto abort;
+	  }
+	  redirect = httpcode;
+	  break;
+
+	case 200 ... 299:
+	  break;
+
+	default:
+	  report_error (location, _("Unexpected HTTP status (%d)"), httpcode);
+	    goto abort;
 	}
-	if (httpcode < 200 || httpcode >= 300) {
-	  report_error (mrl, _("Unexpected HTTP status (%d)"), httpcode);
-	  free (this);
- 	  return NULL;
-	}
-      } else {
-	if (this->contentlength == 0) {
+      }
+      else
+      {
+	if (this->contentlength == 0)
+	{
 	  off_t contentlength;
-	  
-	  if (sscanf(this->buf, "Content-Length: %ld", &contentlength) == 1) {
+	  if (sscanf(this->buf, "Content-Length: %ld", &contentlength) == 1)
+	  {
 	    logprintf ("http: content length = %ld bytes\n", contentlength);
 	    this->contentlength = contentlength;
 	  }
         }
-        if (sscanf(this->buf, "Content-Type: %s", mime_type) == 1) {
+        if (sscanf(this->buf, "Content-Type: %"BUFSIZE_STR"s", mime_type) == 1)
+        {
 	  logprintf ("http: content type = '%s'\n", mime_type);
-	  strcpy(this->mime_type, mime_type);
-	}	
-	if (!strncasecmp(this->buf, "Location: ", 10)) {
-	  /* the server sent an HTTP 'Location:' header */
-	  report_error (mrl, _("HTTP 'Location:' redirection not implemented"));
-	  free (this);
- 	  return NULL;
+	  strcpy (this->mime_type, mime_type); /* note buffer sizes */
+	}
+	if (redirect && !strncasecmp(this->buf, "Location:", 9))
+	{
+	  off_t index = 9;
+	  while (isspace (this->buf[index]))
+	    ++index;
+	  snprintf (this->mrlbuf, sizeof (this->mrlbuf), "%s", this->buf + 10);
+	  logprintf ("http: location = '%s'\n", this->mrlbuf);
+	  close (this->fh); /* heading for disconnect anyway due to protocol */
+	  redirect = 0;
+	  goto retry;
 	}
       }
-      
+
       if (len == -1)
 	done = 1;
       else
@@ -479,6 +567,12 @@ static http_t *http_open (const char *mrl) {
       len ++;
   }
 
+  if (redirect)
+  { /* the server says that the resource is elsewhere but hasn't said where */
+    report_error (location, _("HTTP redirection (%d) is missing a location"), redirect);
+    goto abort;
+  }
+
   return this;
 }
 
@@ -486,23 +580,21 @@ static http_t *http_open (const char *mrl) {
 
 char *http_download (const char *url, ssize_t *file_size)
 {
-  ssize_t total;
-  char   *buf;
-  http_t *http;
-
   logprintf ("http: attempt to download %s\n", url);
 
   if (file_size)
     *file_size = 0;
 
-  http = http_open (url);
+  http_t *http = http_open (url);
   if (!http)
     return NULL;
 
   logprintf ("http: connected\n");
 
-  buf = NULL;  total = 0;
-  while (1) {
+  char   *buf = NULL;
+  ssize_t total = 0;
+  for (;;)
+  {
     int n;
 
     buf = realloc (buf, total + 4098); /* two extra bytes for termination */
@@ -514,7 +606,8 @@ char *http_download (const char *url, ssize_t *file_size)
       break;
     else		/* error */
     {
-      switch (errno) {
+      switch (errno)
+      {
       case EAGAIN:
 	logprintf ("http: EAGAIN\n");
 	continue;
@@ -527,7 +620,7 @@ char *http_download (const char *url, ssize_t *file_size)
     }
   }
 
-  logprintf ("http: got %d bytes of content\n%s", total, buf);
+  logprintf ("http: got %zd bytes of content\n%s", total, buf);
 
   close (http->fh);
 
@@ -549,17 +642,14 @@ char *http_download (const char *url, ssize_t *file_size)
 ssize_t http_peek (const char *url, char *buffer, size_t buffer_size,
 	       char *mime_type, size_t mime_type_size)
 {
-  logprintf ("http: attempt to download %u bytes from %s\n", buffer_size, url);
+  logprintf ("http: attempt to download %zu bytes from %s\n", buffer_size, url);
 
   http_t *http = http_open (url);
   if (!http)
     return 0;
 
   if (mime_type && mime_type_size)
-  {
-    strncpy (mime_type, http->mime_type, mime_type_size);
-    mime_type[mime_type_size - 1] = 0;
-  }
+    snprintf (mime_type, mime_type_size, "%s", http->mime_type);
 
   logprintf ("http: connected\n");
 
