@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2005 the xine-project
+ * Copyright (C) 2002-2007 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,8 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: info_widgets.c,v 1.19 2006/03/27 22:11:24 dsalt Exp $
- *
  * nice black information display areas
  */
 
@@ -35,7 +33,9 @@
 
 #include "i18n.h"
 #include "defs.h"
+#include "gtkvideo.h"
 #include "info_widgets.h"
+#include "ui.h"
 #include "utils.h"
 #include "playlist.h"
 #include "player.h"
@@ -54,8 +54,13 @@ typedef gboolean (*gxineinfo_cb) (gpointer);
 struct gxineinfo_private_s
 {
   gxineinfo_cb postinit, update;
-  GtkWidget *line[MAX_LINES];
+  GtkLabel *line[MAX_LINES];
   int data;
+  struct {
+    int time, length, tcount, tno, ccount, cno;
+    gboolean remaining, seekable;
+  } prev;
+  gboolean force;
 };
 typedef struct gxineinfo_private_s gxineinfo_private_t;
 
@@ -128,7 +133,7 @@ gxineinfo_get_type (void)
     xine_config_register_bool
       (xine, "gui.show_time_remaining", 0,
        _("If set, show the length of the remaining part of the stream."),
-       NULL, 0, gtk_true, NULL);
+       NULL, 0, (xine_config_cb_t) GXINE_TRUE, NULL);
   }
   return type;
 }
@@ -184,9 +189,9 @@ gxineinfo_new (guint height, PangoEllipsizeMode ellipsize, guint width,
   for (i = 0; i < height; ++i)
   {
     GtkWidget *ev = (buttons & 1 << i) ? gtk_event_box_new () : NULL;
-    GtkWidget *line = priv->line[i] = gtk_label_new (NULL);
+    GtkWidget *line = ui_label_new_with_xalign ("", 0);
+    priv->line[i] = GTK_LABEL (line);
     gtk_widget_set_name (line, i ? small : large);
-    gtk_misc_set_alignment (GTK_MISC (line), 0, 0.5);
     gtk_label_set_ellipsize (GTK_LABEL (line), ellipsize);
     if (!i && width)
       gtk_label_set_width_chars (GTK_LABEL (line), width);
@@ -212,7 +217,7 @@ gxineinfo_new (guint height, PangoEllipsizeMode ellipsize, guint width,
  */
 
 void
-gxineinfo_update_line (GSList *objs, guint lineno, const char *format, ...)
+gxineinfo_update_line (const GSList *objs, guint lineno, const char *format, ...)
 {
   gchar *text = NULL;
 
@@ -231,13 +236,13 @@ gxineinfo_update_line (GSList *objs, guint lineno, const char *format, ...)
   {
     gxineinfo_private_t *priv = GET_PRIVATE (objs->data);
     if (priv->line[lineno])
-      gtk_label_set_text (GTK_LABEL (priv->line[lineno]), text);
+      gtk_label_set_text (priv->line[lineno], text);
   }
   free (text);
 }
 
 void
-gxineinfo_update (GSList *objs)
+gxineinfo_update (const GSList *objs)
 {
   foreach_glist (objs, objs)
     if (GTK_IS_GXINE_INFO (objs->data))
@@ -249,7 +254,7 @@ gxineinfo_update (GSList *objs)
 }
 
 void
-gxineinfo_clear (GSList *objs)
+gxineinfo_clear (const GSList *objs)
 {
   int i;
   for (i = 0; i < MAX_LINES; ++i)
@@ -261,7 +266,7 @@ gxineinfo_clear (GSList *objs)
  */
 
 void
-infobar_show_metadata (GSList * bars)
+infobar_show_metadata (const GSList * bars)
 {
   char *strbuf = NULL; /* for the main window's title bar */
   const char *t;
@@ -379,7 +384,8 @@ update_time_widget_cb (gpointer data)
     gchar full[132];
   } time;
 
-  pthread_mutex_lock (&widgets_update_lock);
+  if (pthread_mutex_trylock (&widgets_update_lock))
+    return TRUE;
   gdk_threads_enter ();
 
   time.length[0] = time.current[0] = 0;
@@ -389,11 +395,11 @@ update_time_widget_cb (gpointer data)
 		   !!(xine_config_lookup_entry
 			(xine, "gui.show_time_remaining", &entry)
 		      && entry.num_value);
-  gtk_widget_set_state (widget, state ? GTK_STATE_ACTIVE : GTK_STATE_NORMAL);
+  gboolean seekable = xine_get_stream_info (stream, XINE_STREAM_INFO_SEEKABLE);
+  gtk_widget_set_state ((GtkWidget *)widget, state ? GTK_STATE_ACTIVE : GTK_STATE_NORMAL);
 
   if (xine_get_status (stream) == XINE_STATUS_PLAY && !playlist_showing_logo ())
   {
-
     if (!xine_get_pos_length (stream, &pos_stream, &pos_time, &length_time))
       pos_stream = pos_time = length_time = 0;
 
@@ -401,6 +407,16 @@ update_time_widget_cb (gpointer data)
       pos_time = length_time - pos_time;
 
     pos_time -= pos_time % 1000;
+    if (!priv->force && pos_time == priv->prev.time
+	&& length_time == priv->prev.length && state == priv->prev.remaining
+	&& seekable == priv->prev.seekable)
+      goto finish; /* no need to redisplay the time */
+
+    priv->prev.time = pos_time;
+    priv->prev.length = length_time;
+    priv->prev.remaining = state;
+    priv->prev.seekable = seekable;
+
     int_to_timestring (pos_time, time.current, sizeof (time.current));
 
     length_time = round_second (length_time);
@@ -408,43 +424,57 @@ update_time_widget_cb (gpointer data)
       int_to_timestring (length_time, time.length, sizeof (time.length));
     else
       snprintf (time.length, sizeof (time.length), "%s",
-		xine_get_stream_info (stream, XINE_STREAM_INFO_SEEKABLE)
-		  ? _("??:??:??") : _("live"));
+		seekable ? _("??:??:??") : _("live"));
   }
   else
+  {
+    if (!priv->force && priv->prev.time == -1 && priv->prev.length == -1)
+      goto finish; /* no need to redisplay the (lack of) time */
+    priv->prev.time = priv->prev.length = -1;
     strcpy (time.current, "•");
+  }
+
   if (priv->line[1])
   {
-    gtk_label_set_text (GTK_LABEL (priv->line[0]), time.current);
-    gtk_label_set_text (GTK_LABEL (priv->line[1]), time.length);
+    gtk_label_set_text (priv->line[0], time.current);
+    gtk_label_set_text (priv->line[1], time.length);
   }
   else if (time.current[0])
   {
     strcat (time.full, " / ");
     memmove (time.full + strlen (time.full), time.length, sizeof (time.length));
-    gtk_label_set_text (GTK_LABEL (priv->line[0]), time.full);
+    gtk_label_set_text (priv->line[0], time.full);
   }
   else
-    gtk_label_set_text (GTK_LABEL (priv->line[0]), "");
+    gtk_label_set_text (priv->line[0], "");
 
+finish:
 #ifdef XINE_STREAM_INFO_DVD_TITLE_NUMBER
   time.full[0] = 0;
-  int count = xine_get_stream_info (stream, XINE_STREAM_INFO_DVD_TITLE_COUNT);
-  if (count) /* "T" = DVD title number */
-    sprintf (time.full, _("T%d/%d "),
-	     xine_get_stream_info (stream, XINE_STREAM_INFO_DVD_TITLE_NUMBER),
-	     count);
-  count = xine_get_stream_info (stream, XINE_STREAM_INFO_DVD_CHAPTER_COUNT);
-  if (count) /* "C" = DVD chapter number */
-    sprintf (time.full + strlen (time.full), _("C%d/%d "),
-	     xine_get_stream_info (stream, XINE_STREAM_INFO_DVD_CHAPTER_NUMBER),
-	     count);
-  gtk_label_set_text (GTK_LABEL (priv->line[2]), time.full);
+  int tcount = xine_get_stream_info (stream, XINE_STREAM_INFO_DVD_TITLE_COUNT);
+  int tno = tcount ? xine_get_stream_info (stream, XINE_STREAM_INFO_DVD_TITLE_NUMBER) : 0;
+  int ccount = xine_get_stream_info (stream, XINE_STREAM_INFO_DVD_CHAPTER_COUNT);
+  int cno = ccount ? xine_get_stream_info (stream, XINE_STREAM_INFO_DVD_CHAPTER_NUMBER) : 0;
+
+  if (priv->force || tcount != priv->prev.tcount || tno != priv->prev.tno
+		  || ccount != priv->prev.ccount || cno != priv->prev.cno)
+  {
+    priv->prev.tcount = tcount;
+    priv->prev.tno = tno;
+    priv->prev.ccount = ccount;
+    priv->prev.cno = cno;
+    if (tcount) /* "T" = DVD title number */
+      sprintf (time.full, _("T%d/%d "), tno, tcount);
+    if (ccount) /* "C" = DVD chapter number */
+      sprintf (time.full + strlen (time.full), _("C%d/%d "), cno, ccount);
+    gtk_label_set_text (priv->line[2], time.full);
+  }
 #endif
 
+  priv->force = FALSE;
   gdk_threads_leave ();
   pthread_mutex_unlock (&widgets_update_lock);
-  return TRUE;
+  return FALSE;
 }
 
 static gboolean
@@ -452,8 +482,25 @@ frob_format_cb (GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
   if (event->button == 1)
   {
-    GET_PRIVATE (data)->data ^= 1;
-    update_time_widget_cb (data);
+    gxineinfo_private_t *priv = GET_PRIVATE (data);
+    int flag = (priv->data ^= 1);
+    priv->force = TRUE;
+
+    GSList **listp = g_object_get_data ((GObject *) data, "gxine-chain");
+    if (listp)
+    {
+      GSList *item = *listp;
+      while (item)
+      {
+	priv = GET_PRIVATE (item->data);
+	priv->data = flag;
+	priv->force = TRUE;
+	item = item->next;
+      }
+      gxineinfo_update (*listp);
+    }
+    else
+      update_time_widget_cb (data);
   }
   return TRUE;
 }
@@ -462,23 +509,27 @@ static gboolean
 postinit_time_widget (gpointer widget)
 {
   gxineinfo_private_t *priv = GET_PRIVATE (widget);
-  g_signal_connect (G_OBJECT (priv->line[0]->parent), "button-press-event",
+  GtkWidget *line = GTK_WIDGET (priv->line[0]);
+  g_signal_connect (G_OBJECT (line->parent), "button-press-event",
 		    G_CALLBACK (frob_format_cb), widget);
 
-  GdkWindow *window = priv->line[0]->window;
+  GdkWindow *window = line->window;
   GdkDisplay *display = gdk_drawable_get_display ((GdkDrawable *)window);
   GdkCursor *ptr = gdk_cursor_new_for_display (display, GDK_CLOCK);
   gdk_window_set_cursor (window, ptr);
   gdk_cursor_unref (ptr);
 
-  g_timeout_add (500, update_time_widget_cb, widget);
   return FALSE;
 }
 
 GtkWidget *
-create_time_widget (gboolean small)
+create_time_widget (gboolean small, const GSList *const *const listp)
 {
-  return gxineinfo_new (small ? 1 : 3, PANGO_ELLIPSIZE_NONE, small ? 16 : 8,
-			postinit_time_widget, update_time_widget_cb, "time",
-			1 /* line 0 has an event box */);
+  GtkWidget *w = gxineinfo_new (small ? 1 : 3, PANGO_ELLIPSIZE_NONE,
+				small ? 16 : 8, postinit_time_widget,
+				update_time_widget_cb, "time",
+				1 /* line 0 has an event box */);
+  if (listp)
+    g_object_set_data ((GObject *) w, "gxine-chain", (gpointer) listp);
+  return w;
 }
