@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2004 the xine-project
+ * Copyright (C) 2002-2007 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,8 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: mediamarks.c,v 1.69 2006/04/08 21:34:50 dsalt Exp $
- *
  * mediamarks - load
  *            - save
  *            - edit
@@ -39,11 +37,13 @@
 #include "engine.h"
 #include "mediamarks.h"
 #include "menu.h"
+#include "list_menus.h"
 #include "playlist.h"
 #include "play_item.h"
 
 static GtkTreeStore *mm_store;
 static GtkTreeModel *mm_model;
+static GtkTreeView  *tree;
 static GtkTreeSelection *sel;
 static GtkWidget    *manage_dlg, *cat_dlg, *cat_entry, *mm_tree;
 static GtkTreeIter   cat_iter;
@@ -51,6 +51,8 @@ static gboolean      is_visible, cat_edited;
 
 static GtkUIManager *mm_ui;
 static GtkActionGroup *media_group = NULL;
+static GtkAction      *play_item;
+static GtkMenu	      *menu[2];
 static guint	       media_merge;
 
 #define COLUMN_TITLE		0
@@ -78,6 +80,18 @@ static inline play_item_t *peek_play_item (GtkTreeIter *iter)
   return p;
 }
 
+static void play_cb (GtkWidget* widget, gpointer data)
+{
+  GtkTreeIter iter;
+
+  if (gtk_tree_selection_get_selected (sel, NULL, &iter))
+  {
+    play_item_t *item = peek_play_item (&iter);
+    if (item)
+      playlist_play (playlist_add (play_item_copy (item), -1));
+  }
+}
+
 static void menu_cb (GtkAction* action, gpointer data)
 {
   if (data)
@@ -104,12 +118,11 @@ static int gen_actions_sub (GtkUIManager *ui, GtkActionGroup *actions,
       else
       {
 	GtkTreeIter children;
-	if (gtk_tree_model_iter_children (GTK_TREE_MODEL(mm_store),
-					  &children, iter))
+	if (gtk_tree_model_iter_children (mm_model, &children, iter))
 	  count = gen_actions_sub (ui, actions, &children, count);
       }
     }
-  } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(mm_store), iter));
+  } while (gtk_tree_model_iter_next (mm_model, iter));
 
   return count;
 }
@@ -141,8 +154,7 @@ static int gen_items (GtkUIManager *ui, const char *path, GtkTreeIter *iter,
 	GtkTreeIter children;
 	gtk_ui_manager_add_ui (ui, merge, path, name, name,
 			       GTK_UI_MANAGER_MENU, FALSE);
-	if (gtk_tree_model_iter_children (GTK_TREE_MODEL(mm_store),
-					  &children, iter))
+	if (gtk_tree_model_iter_children (mm_model, &children, iter))
 	{
 	  char *newpath = g_strdup_printf ("%s/%s", path, name);
 	  count = gen_items (ui, newpath, &children, merge, count);
@@ -150,7 +162,7 @@ static int gen_items (GtkUIManager *ui, const char *path, GtkTreeIter *iter,
 	}
       }
     }
-  } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(mm_store), iter));
+  } while (gtk_tree_model_iter_next (mm_model, iter));
 
   return count;
 }
@@ -200,7 +212,7 @@ static void get_items (xml_node_t *node, int depth, GtkTreeIter *parent)
     if (!strcasecmp (node->name, "sub"))
     {
       GtkTreeIter iter;
-      char *id = xml_parser_get_property (node, "name");
+      const char *id = xml_parser_get_property (node, "name");
 
       gtk_tree_store_append (mm_store, &iter, parent);
       gtk_tree_store_set (mm_store, &iter, COLUMN_TITLE, id, -1);
@@ -314,7 +326,7 @@ static void load_old_file (char *fname)
         }
 
 	sscanf (str, "%d", &time);
-	play_item = play_item_new (id, mrl, time);
+	play_item = play_item_new (id, mrl, time, 0);
 
 	gtk_tree_store_append (mm_store, &iter, depth ? &parent : NULL);
 	gtk_tree_store_set (mm_store, &iter,
@@ -396,6 +408,18 @@ static void edit_cb (GtkWidget* widget, gpointer data)
   }
 }
 
+static gboolean row_clicked_lcb (GtkWidget *widget, GdkEventButton *ev,
+				 gpointer data)
+{
+  return list_menu_click_cb (tree, ev, (GtkMenu **) data);
+}
+
+static gboolean key_press_lcb (GtkWidget *widget, GdkEventKey *ev,
+				 gpointer data)
+{
+  return list_menu_key_press_cb (tree, ev, (GtkMenu **) data);
+}
+
 static void delete_cb (GtkWidget* widget, gpointer data)
 {
   GtkTreeIter iter;
@@ -527,7 +551,7 @@ void mm_add_show (void *data)
   char	      *dot;
   const char  *title = xine_get_meta_info (stream, XINE_META_INFO_TITLE);
 
-  play_item = playlist_get_current_item () ? : play_item_new ("", "", 0);
+  play_item = playlist_get_current_item () ? : play_item_new ("", "", 0, 0);
 
   /* default title? try to get title from meta-info */
   dot = strrchr (play_item->mrl, '.');
@@ -623,7 +647,7 @@ static void save_items (FILE *f, int depth, GtkTreeIter *iter)
 	print_depth (f, depth);
 	fprintf (f, "<SUB NAME=\"%s\">\n", id);
 
-	if (gtk_tree_model_iter_children (GTK_TREE_MODEL(mm_store), &children, iter))
+	if (gtk_tree_model_iter_children (mm_model, &children, iter))
 	  save_items (f, depth+2, &children);
 
 	print_depth (f, depth);
@@ -631,7 +655,7 @@ static void save_items (FILE *f, int depth, GtkTreeIter *iter)
       }
     }
 
-  } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(mm_store), iter));
+  } while (gtk_tree_model_iter_next (mm_model, iter));
 }
 
 void mm_save (void)
@@ -696,9 +720,17 @@ static void mm_response_cb (GtkDialog *dbox, int response, gpointer data)
   }
 }
 
+static gboolean map_cb (GtkWidget* widget, gpointer data)
+{
+  gdk_window_set_events (widget->window,
+			 gdk_window_get_events (widget->window) | GDK_BUTTON_PRESS_MASK);
+  return FALSE;
+}
+
 static const GtkActionEntry buttons_data[] = {
+  { "play",	GTK_STOCK_MEDIA_PLAY,	NULL,			NULL,		NULL,				play_cb },
   { "new",	GTK_STOCK_NEW,		NULL,			NULL,		N_("Add an MRL"),		mm_add_show },
-  { "newcat",	GTK_STOCK_DIRECTORY,	N_("New category"),	"<Control>F",	N_("Add a category folder"),	new_category_cb },
+  { "newcat",	GTK_STOCK_DIRECTORY,	N_("_Category"),	"<Control>F",	N_("Add a category folder"),	new_category_cb },
   { "edit",	GTK_STOCK_EDIT,		NULL,			"<Control>E",	N_("Edit the selected item"),	edit_cb },
   { "delete",	GTK_STOCK_DELETE,	NULL,			"Delete",	N_("Delete the selected item"),	delete_cb },
   { "copy",	GTK_STOCK_COPY,		NULL,			NULL,		NULL,				copy_cb },
@@ -716,13 +748,29 @@ static const char buttons_structure[] =
       "<toolitem action='copy' />\n"
       "<toolitem action='cut' />\n"
       "<toolitem action='paste' />\n"
+      "<separator />\n"
+      "<toolitem action='play' />\n"
     "</toolbar>\n"
+    "<popup name='item_menu'>\n"
+      "<menuitem action='play' />\n"
+      "<separator />\n"
+      "<menuitem action='edit' />\n"
+      "<menuitem action='delete' />\n"
+      "<menuitem action='copy' />\n"
+      "<menuitem action='cut' />\n"
+      "<menuitem action='paste' />\n"
+    "</popup>\n"
+    "<popup name='list_menu'>\n"
+      "<menuitem action='new' />\n"
+      "<menuitem action='newcat' />\n"
+      "<menuitem action='paste' />\n"
+    "</popup>\n"
   "</ui>";
 
 static void sel_changed_cb (GtkTreeSelection *sel, gpointer data)
 {
   static const char *const items[] = { "edit", "delete", NULL };
-  static const char *const cb_items[] = { "copy", "cut", NULL };
+  static const char *const cb_items[] = { "play", "copy", "cut", NULL };
   GtkTreeIter iter;
   gboolean have_selected = gtk_tree_selection_get_selected (sel, NULL, &iter);
 
@@ -735,12 +783,41 @@ static void sel_changed_cb (GtkTreeSelection *sel, gpointer data)
 
 void mediamarks_init (void)
 {
-  GtkWidget            *tree, *scrolled_window, *hbox, *label;
+  GtkWidget	       *scrolled_window, *hbox, *label;
   GtkBox	       *vbox;
   gchar                *fname;
   GtkCellRenderer      *cell;
   GtkTreeViewColumn    *column;
-  GError	       *error;
+  GError	       *error = NULL;
+
+  /*
+   * create (for now invisible) mm_manage dialog
+   */
+
+  manage_dlg = gtk_dialog_new_with_buttons (_("Manage media marks..."), NULL, 0,
+				GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+				GTK_STOCK_CLOSE, GTK_RESPONSE_DELETE_EVENT,
+				NULL);
+  gtk_window_set_default_size (GTK_WINDOW (manage_dlg), 400, 250);
+  hide_on_delete (manage_dlg, &is_visible);
+  vbox = GTK_BOX (GTK_DIALOG (manage_dlg)->vbox);
+
+  /* buttons & menus */
+
+  mm_ui = ui_create_manager ("mm", manage_dlg);
+  GtkActionGroup *group = ui_get_action_group (mm_ui);
+  gtk_action_group_add_actions (group, buttons_data,
+				G_N_ELEMENTS (buttons_data), tree);
+  gtk_ui_manager_add_ui_from_string (mm_ui, buttons_structure, -1, &error);
+  if (error)
+  {
+    g_printerr (_("playlist XML: %s\n"), error->message);
+    g_clear_error (&error);
+  }
+  menu[0] = GTK_MENU (gtk_ui_manager_get_widget (mm_ui, "/ui/list_menu"));
+  menu[1] = GTK_MENU (gtk_ui_manager_get_widget (mm_ui, "/ui/item_menu"));
+  play_item = gtk_action_group_get_action (group, "play");
+  gtk_action_group_connect_accelerators (group);
 
   /*
    * init tree store
@@ -756,39 +833,27 @@ void mediamarks_init (void)
 	"signal::rows-reordered", G_CALLBACK (tree_changed_cb), NULL,
 	NULL);
 
-  /*
-   * create (for now invisible) mm_manage dialog
-   */
+  /* connect up the signals now that the menus etc. are initialised */
 
-  manage_dlg = gtk_dialog_new_with_buttons (_("Manage media marks..."), NULL, 0,
-				GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-				GTK_STOCK_CLOSE, GTK_RESPONSE_DELETE_EVENT,
-				NULL);
-  gtk_window_set_default_size (GTK_WINDOW (manage_dlg), 400, 250);
-  g_signal_connect (G_OBJECT(manage_dlg), "response",
-		    G_CALLBACK(mm_response_cb), NULL);
-  hide_on_delete (manage_dlg, &is_visible);
-
-  vbox = GTK_BOX (GTK_DIALOG (manage_dlg)->vbox);
+  g_object_connect (G_OBJECT(manage_dlg),
+	"signal::map-event", G_CALLBACK (map_cb), NULL,
+	"signal::button-press-event", G_CALLBACK(list_menu_popup_cb), menu[0],
+	"signal::key-press-event", G_CALLBACK(key_press_lcb), menu,
+	"signal::response", G_CALLBACK(mm_response_cb), NULL,
+	NULL);
 
   /* add a nice toolbar and tree view widget here */
 
-  mm_tree = tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL(mm_store));
-  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tree), TRUE);
+  mm_tree = gtk_tree_view_new_with_model (mm_model);
+  tree = GTK_TREE_VIEW (mm_tree);
+  gtk_tree_view_set_rules_hint (tree, TRUE);
 
-  sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
+  sel = gtk_tree_view_get_selection (tree);
   g_signal_connect (G_OBJECT (sel), "changed",
 		    G_CALLBACK (sel_changed_cb), NULL);
 
-  /* buttons */
-
-  mm_ui = ui_create_manager ("mm", manage_dlg);
-  gtk_action_group_add_actions (ui_get_action_group (mm_ui), buttons_data,
-				G_N_ELEMENTS (buttons_data), tree);
-  gtk_ui_manager_add_ui_from_string (mm_ui, buttons_structure, -1, &error);
-  gtk_box_pack_start (vbox, gtk_ui_manager_get_widget (mm_ui, "/toolbar"),
+  gtk_box_pack_start (vbox, gtk_ui_manager_get_widget (mm_ui, "/ui/toolbar"),
 		      FALSE, FALSE, 0);
-  gtk_action_group_connect_accelerators (ui_get_action_group (mm_ui));
 
   sel_changed_cb (sel, NULL);
 
@@ -799,30 +864,29 @@ void mediamarks_init (void)
 		"ellipsize", 2, "width_chars", 16, NULL); /* foo...bar */
   column = gtk_tree_view_column_new_with_attributes (_("Media mark"), cell,
 						     "text", COLUMN_TITLE, NULL);
-  gtk_tree_view_column_set_resizable (GTK_TREE_VIEW_COLUMN(column), TRUE);
-  gtk_tree_view_append_column (GTK_TREE_VIEW (tree),
-			       GTK_TREE_VIEW_COLUMN (column));
+  gtk_tree_view_column_set_resizable (column, TRUE);
+  gtk_tree_view_append_column (tree, column);
 
   cell = gtk_cell_renderer_text_new ();
   g_object_set (G_OBJECT(cell),
 		"ellipsize", 1, "width_chars", 16, NULL); /* ...bar */
   column = gtk_tree_view_column_new_with_attributes (_("MRL"), cell,
 						     "text", COLUMN_MRL, NULL);
-  gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN(column),
-				   GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-  gtk_tree_view_append_column (GTK_TREE_VIEW (tree),
-			       GTK_TREE_VIEW_COLUMN (column));
+  gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+  gtk_tree_view_append_column (tree, column);
 
-  gtk_tree_view_set_reorderable (GTK_TREE_VIEW (tree), TRUE);
+  gtk_tree_view_set_reorderable (tree, TRUE);
 
   scrolled_window = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
 				  GTK_POLICY_AUTOMATIC,
 				  GTK_POLICY_AUTOMATIC);
-  gtk_container_add (GTK_CONTAINER (scrolled_window), tree);
+  gtk_container_add (GTK_CONTAINER (scrolled_window), mm_tree);
 
-  g_signal_connect (G_OBJECT (tree), "row-activated",
-		    G_CALLBACK (edit_cb), NULL);
+  g_object_connect (G_OBJECT (tree),
+	"signal::row-activated", G_CALLBACK (edit_cb), NULL,
+	"signal::button-press-event", G_CALLBACK (row_clicked_lcb), menu,
+	NULL);
 
   gtk_box_pack_start (vbox, scrolled_window, TRUE, TRUE, 2);
 
