@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2003 the xine project
+ * Copyright (C) 2001-2006 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,12 +15,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: mediamarks.h,v 1.9 2006-02-21 19:06:12 dsalt Exp $
- *
  */
 
-#ifndef HAVE_MEDIAMARKS_H
-#define HAVE_MEDIAMARKS_H
+#ifndef GXINE_MEDIAMARKS_H
+#define GXINE_MEDIAMARKS_H
 
 #include "play_item.h"
 
