@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2005 the xine project
+ * Copyright (C) 2003-2006 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,13 +15,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: menu.h,v 1.8 2006-02-21 19:06:12 dsalt Exp $
- *
  * main/context menu creation / handling
  */
 
-#ifndef HAVE_MENU_H
-#define HAVE_MENU_H
+#ifndef GXINE_MENU_H
+#define GXINE_MENU_H
 
 #include <gtk/gtkwidget.h>
 
@@ -34,6 +32,8 @@ const char *menu_get_tree_name (unsigned int);
 void scale_changed_cb (GtkWidget *, gdouble, gpointer);
 GtkAction *find_video_size_action (gdouble);
 
+void update_subtitles_menu (void);
+
 extern GtkUIManager *ui;
 
 #endif
