@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2004 the xine project
+ * Copyright (C) 2003-2007 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,8 +15,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: noskin_window.c,v 1.108 2006/04/08 21:34:50 dsalt Exp $
- *
  * standard, non-skinned main window
  */
 
@@ -59,8 +57,9 @@ GtkWidget *gtv;
 static GtkTooltips    *tips;
 static int             have_video = -1, have_audio = -1;
 static struct app_prop_s {
-  gboolean	       sticky, fs, real_sticky;
-} app_prop = { FALSE, FALSE, FALSE };
+  gboolean	       sticky, fs, real_sticky, shaded;
+  signed char          position; /* <0 = below, >0 = above */
+} app_prop = { FALSE, FALSE, FALSE, FALSE, 0 };
 
 static GtkWidget *fs_toolbar, *wm_toolbar = NULL;
 static Atom opacity_lock;
@@ -175,6 +174,9 @@ static gboolean update_slider_cb (gpointer data)
 {
   gint pos_stream, pos_time, length_time;
 
+  if (pthread_mutex_trylock (&widgets_update_lock))
+    return TRUE;
+
   window_check_vis (FALSE);
   if (xine_get_status (stream) == XINE_STATUS_PLAY
       && xine_get_pos_length (stream, &pos_stream, &pos_time, &length_time))
@@ -184,6 +186,7 @@ static gboolean update_slider_cb (gpointer data)
     ui_set_status (UI_CURRENT_STATE);
   }
 
+  pthread_mutex_unlock (&widgets_update_lock);
   return TRUE;
 }
 
@@ -235,7 +238,8 @@ static gboolean window_fs_toolbar_show_int (void)
   window_fs_toolbar_set_geometry ();
   window_fs_toolbar_move (NULL);
   window_show (fs_toolbar, NULL);
-  gtk_window_stick (GTK_WINDOW(fs_toolbar));
+  if (fs_always_sticky)
+    gtk_window_stick (GTK_WINDOW(fs_toolbar));
   XChangeProperty (GDK_WINDOW_XDISPLAY (fs_toolbar->window),
 		   GDK_WINDOW_XWINDOW (fs_toolbar->window), opacity_lock,
 		   XA_CARDINAL, 32, PropModeReplace,
@@ -318,7 +322,7 @@ void window_wm_toolbar_reset (void)
 
 void window_wm_toolbar_restore (void)
 {
-  if (wm_toolbar)
+  if (wm_toolbar && GTK_WIDGET_MAPPED (app))
     gtk_window_present (GTK_WINDOW (wm_toolbar));
 }
 
@@ -334,6 +338,19 @@ void window_wm_toolbar_set_snap (gboolean snap)
 
 /* callbacks */
 
+void window_fs_stickiness_update (gboolean data)
+{
+  fs_always_sticky = data;
+
+  if (gtk_video_is_fullscreen ((GtkVideo *) gtv))
+  {
+    if (data || app_prop.sticky)
+      gtk_window_stick ((GtkWindow *) app);
+    else
+      gtk_window_unstick ((GtkWindow *) app);
+  }
+}
+
 #ifdef WINDOW_STACKING_HACK
 /* whether gdk_window_lower() is used; present in case the wm toolbar window
  * should somehow gain the input focus, since calling this may move the focus
@@ -363,29 +380,34 @@ gboolean window_wm_toolbar_snap (void)
   gdk_window_get_frame_extents (app->window, &appframe);
   gdk_window_get_frame_extents (wm_toolbar->window, &tbframe);
 
+  GdkRectangle monitor;
+  gdk_screen_get_monitor_geometry (((GtkWindow *)app)->screen,
+				   gdk_screen_get_monitor_at_window
+				     (((GtkWindow *)app)->screen, app->window),
+				   &monitor);
+
   /* centre horizontally wrt video window */
   x = appframe.x + (appframe.width - tbframe.width) / 2;
   /* ensure fully on-screen unless the video window isn't fully on-screen */
   if (appframe.width >= tbframe.width)
   {
-    if (x < 0)
-      x = MIN (0, appframe.x + appframe.width - tbframe.width);
-    else if (x + tbframe.width >
-	     (y = gdk_screen_get_width (((GtkWindow *)app)->screen)))
-      x = MAX (y - tbframe.width, appframe.x);
+    if (x < monitor.x)
+      x = MIN (monitor.x, appframe.x + appframe.width - tbframe.width);
+    else if (x + tbframe.width > monitor.x + monitor.width)
+      x = MAX (monitor.x + monitor.width - tbframe.width, appframe.x);
   }
   else
   {
-    if (x < 0)
-      x = MIN (0, appframe.x);
-    else if (x + tbframe.width >
-	     (y = gdk_screen_get_width (((GtkWindow *)app)->screen)))
-      x = MAX (y - tbframe.width, appframe.x + appframe.width - tbframe.width);
+    if (x < monitor.x)
+      x = MIN (monitor.x, appframe.x);
+    else if (x + tbframe.width > monitor.x + monitor.width)
+      x = MAX (monitor.x + monitor.width - tbframe.width,
+	       appframe.x + appframe.width - tbframe.width);
   }
 
   /* position just below the video window, or above if insufficient space */
   y = appframe.y + appframe.height;
-  if (y + tbframe.height > gdk_screen_get_height (((GtkWindow *)app)->screen))
+  if (y + tbframe.height > monitor.y + monitor.height)
     y = appframe.y - tbframe.height;
 
   /* reposition if needed */
@@ -427,7 +449,7 @@ static void fs_show_hide (GtkWidget *widget, gpointer show)
 
 static gboolean refocus_cb (GtkWidget *widget)
 {
-  if (GTK_WIDGET_VISIBLE (widget))
+  if (GTK_WIDGET_VISIBLE (widget) && !app_prop.shaded)
     XSetInputFocus (GDK_WINDOW_XDISPLAY (widget->window),
 		    GDK_WINDOW_XID (widget->window),
 		    RevertToParent, CurrentTime);
@@ -453,13 +475,17 @@ app_state_cb (GtkWindow *window, GdkEventWindowState *event, gpointer data)
     gtk_video_deny_shrink ((GtkVideo *) gtv);
 
     /* restore stickiness as was, pre-fullscreen */
-    if (app_prop.sticky)
-      gtk_window_stick (window);
-    else
-      gtk_window_unstick (window);
+    if (fs_always_sticky)
+    {
+      if (app_prop.sticky)
+	gtk_window_stick (window);
+      else
+	gtk_window_unstick (window);
+    }
 
     /* restore handling wrt window stack */
-    gtk_window_set_keep_above (window, FALSE);
+    gtk_window_set_keep_below (window, app_prop.position < 0);
+    gtk_window_set_keep_above (window, app_prop.position > 0);
 
     /* reshow the toolbar (if needed) */
     if (!wm_toolbar)
@@ -487,7 +513,8 @@ app_state_cb (GtkWindow *window, GdkEventWindowState *event, gpointer data)
   else
   {
     gtk_window_set_shade (window, FALSE);
-    gtk_window_stick (window);
+    if (fs_always_sticky)
+      gtk_window_stick (window);
     if (!wm_toolbar)
       gtk_container_foreach (GTK_CONTAINER (window->bin.child),
 			     fs_show_hide, NULL);
@@ -503,6 +530,20 @@ app_state_cb (GtkWindow *window, GdkEventWindowState *event, gpointer data)
   return FALSE;
 }
 
+void app_show (void)
+{
+  gtk_widget_show (app);
+  if (wm_toolbar && wm_toolbar_visible)
+    gtk_widget_show (wm_toolbar);
+}
+
+void app_hide (void)
+{
+  gtk_widget_hide (app);
+  if (wm_toolbar && wm_toolbar_visible)
+    gtk_widget_hide (wm_toolbar);
+}
+
 static void
 xevent_properties (void)
 {
@@ -530,6 +571,11 @@ xevent_properties (void)
     gdk_x11_get_xatom_by_name_for_display (d, "_NET_WM_STATE_STICKY");
   Atom fs =
     gdk_x11_get_xatom_by_name_for_display (d, "_NET_WM_STATE_FULLSCREEN");
+  Atom shaded =
+    gdk_x11_get_xatom_by_name_for_display (d, "_NET_WM_STATE_SHADED");
+  Atom above = gdk_x11_get_xatom_by_name_for_display (d, "_NET_WM_STATE_ABOVE");
+  Atom below = gdk_x11_get_xatom_by_name_for_display (d, "_NET_WM_STATE_BELOW");
+
   Atom *atoms = (Atom *) prop;
   unsigned long i;
   for (i = 0; i < items; ++i)
@@ -537,9 +583,18 @@ xevent_properties (void)
       new_prop.real_sticky = TRUE;
     else if (atoms[i] == fs)
       new_prop.fs = TRUE;
+    else if (atoms[i] == shaded)
+      new_prop.shaded = TRUE;
+    else if (atoms[i] == above)
+      ++new_prop.position;
+    else if (atoms[i] == below)
+      --new_prop.position;
 
   XFree (prop);
 
+  if (gtk_video_is_fullscreen ((GtkVideo *)gtv))
+    new_prop.position = app_prop.position; /* will be on top regardless */
+
   if (new_prop.real_sticky)
   {
     /* sticky only if "sticky" property set & window on all desktops */
@@ -555,7 +610,7 @@ xevent_properties (void)
 
   if (app_prop.real_sticky != new_prop.real_sticky)
   {
-    if (!new_prop.fs)
+    if (!new_prop.fs || !fs_always_sticky)
       new_prop.sticky = new_prop.real_sticky;
     else if (!new_prop.real_sticky)
     {
@@ -565,6 +620,9 @@ xevent_properties (void)
     }
   }
 
+  if (app_prop.shaded != new_prop.shaded && wm_toolbar_snap)
+    g_idle_add ((GSourceFunc) window_wm_toolbar_snap, NULL);
+
   app_prop = new_prop;
 
 unchanged:
@@ -587,19 +645,22 @@ xevent_filter_cb (GdkXEvent *gx, GdkEvent *e, gpointer d)
     return GDK_FILTER_REMOVE; /* else the screen goes black :-| */
 
   case FocusOut:
-    if (gtk_video_is_fullscreen ((GtkVideo *) gtv))
+    if (gtk_video_is_fullscreen ((GtkVideo *) gtv)
+	&& GTK_WIDGET_MAPPED (gtv))
     {
       Display *xd = GDK_WINDOW_XDISPLAY (((GtkWidget *) d)->window);
       Window fw = None;
       int fr;
+      gdk_error_trap_push ();
       XGetInputFocus (xd, &fw, &fr);
       GdkWindow *gw = gdk_window_lookup (fw);
 #ifdef HAVE_XINERAMA
-      if (gw == fs_toolbar->window || (!gw && !XineramaIsActive (xd)))
+      if (gw ? gw == fs_toolbar->window : !XineramaIsActive (xd))
 #else
       if (!gw || gw == fs_toolbar->window)
 #endif
 	XSetInputFocus (xd, x->xany.window, RevertToParent, CurrentTime);
+      gdk_error_trap_pop ();
     }
     return GDK_FILTER_REMOVE;
 
@@ -612,17 +673,8 @@ xevent_filter_cb (GdkXEvent *gx, GdkEvent *e, gpointer d)
   return GDK_FILTER_CONTINUE;
 }
 
-static gboolean noskin_main_init_cb (void)
+static gboolean noskin_main_init_cb (gpointer fullscreen)
 {
-  xine_cfg_entry_t entry;
-  if (xine_config_lookup_entry (xine, "gui.window_size", &entry))
-  {
-    static const gdouble scale[] = { 50, 75, 100, 150, 200 };
-    gtk_video_rescale ((GtkVideo *)gtv, scale[entry.num_value]);
-  }
-  if (xine_config_lookup_entry (xine, "gui.magnify_lowres_video", &entry))
-    gtk_video_set_auto_rescale ((GtkVideo *)gtv, entry.num_value);
-
   gtk_widget_add_events (app, GDK_KEY_RELEASE_MASK);
   g_object_connect (G_OBJECT (app),
 	"signal::key-press-event", G_CALLBACK (keypress_cb), wm_toolbar,
@@ -640,11 +692,18 @@ static gboolean noskin_main_init_cb (void)
   gdk_window_set_events (app->window, GDK_ALL_EVENTS_MASK); /* seems to work */
   gdk_window_add_filter (app->window, xevent_filter_cb, app);
   gtk_widget_grab_focus (app);
+
+  gtk_video_set_frame_cb ((GtkVideo *)gtv, (gtk_video_frame_cb_t)gxineinfo_update, timewidgets);
+
+  if (fullscreen)
+    gtk_window_fullscreen (GTK_WINDOW (app));
+
 /*
   XSetInputFocus (GDK_WINDOW_XDISPLAY (app->window),
 		  GDK_WINDOW_XWINDOW (app->window),
 		  RevertToParent, CurrentTime);
 */
+  gxine_init_done ();
   return FALSE;
 }
 
@@ -699,8 +758,23 @@ wm_show_hide_setup (gpointer object)
 		    NULL);
 }
 
+static gboolean noskin_post_init (gboolean unused)
+{
+  xine_cfg_entry_t entry;
+  if (xine_config_lookup_entry (xine, "gui.window_size", &entry))
+  {
+    static const gdouble scale[] = { 50, 75, 100, 150, 200 };
+    gtk_video_rescale ((GtkVideo *)gtv, scale[entry.num_value]);
+  }
+  if (xine_config_lookup_entry (xine, "gui.windowed_mode.magnify_lowres_video", &entry))
+    gtk_video_set_auto_rescale ((GtkVideo *)gtv, entry.num_value);
 
-void noskin_main_init (const gchar *geometry)
+  gxine_init_done ();
+  return FALSE;
+}
+
+
+void noskin_main_init (const gchar *geometry, gboolean fullscreen)
 {
   GtkWidget     *vbox, *w;
   GtkWindow     *W;
@@ -715,7 +789,7 @@ void noskin_main_init (const gchar *geometry)
   fs_toolbar = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_widget_set_name (fs_toolbar, "fs_toolbar");
 
-  if (!xine_config_lookup_entry (xine, "gui.windowedmode_separate_toolbar",
+  if (!xine_config_lookup_entry (xine, "gui.windowed_mode.separate_toolbar",
 				 &entry)
       || entry.num_value)
   {
@@ -774,6 +848,10 @@ void noskin_main_init (const gchar *geometry)
   	               video_driver_id, 320, 240,
                        0x04 /* press: button 2 */,
                        0x08 /* release: button 3 */);
+  if (xine_config_lookup_entry (xine, "gui.windowed_mode.auto_resize", &entry))
+    gtk_video_set_auto_resize ((GtkVideo *)gtv, entry.num_value);
+
+  gxine_init_add ((GSourceFunc) noskin_post_init, NULL);
 
   if (!wm_toolbar)
   {
@@ -885,6 +963,8 @@ void noskin_main_init (const gchar *geometry)
     gtk_widget_show_all (wm_toolbar);
     XSelectInput (GDK_WINDOW_XDISPLAY (app->window), GDK_WINDOW_XID (app->window),
 		  VisibilityChangeMask);
+    if (fullscreen)
+      gtk_widget_hide (wm_toolbar);
   }
 
   g_object_set (G_OBJECT (app), "allow-shrink", TRUE, NULL);
@@ -910,6 +990,6 @@ void noskin_main_init (const gchar *geometry)
 		   XA_CARDINAL, 32, PropModeReplace,
 		   (unsigned char *)&have_video, 1);
 
-  gtk_init_add ((GSourceFunc) noskin_main_init_cb, NULL);
+  gxine_init_add ((GSourceFunc) noskin_main_init_cb, GINT_TO_POINTER (fullscreen));
   g_timeout_add (500, update_slider_cb, NULL);
 }
