@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2004 the xine project
+ * Copyright (C) 2003-2006 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,14 +15,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: play_item.h,v 1.15 2006-03-05 23:57:01 dsalt Exp $
- *
  * playlist item / media mark:
  * an mrl + options (e.g. volume, start time, brightness/contrast ...)
  */
 
-#ifndef HAVE_PLAY_ITEM_H
-#define HAVE_PLAY_ITEM_H
+#ifndef GXINE_PLAY_ITEM_H
+#define GXINE_PLAY_ITEM_H
 
 #include <glib.h>
 #include <stdio.h>
@@ -42,7 +40,11 @@ struct play_item_s {
   char   *title;
   char   *mrl;
 
-  int     start_time;
+  int	  start_time, duration;
+  struct {
+    int offset;
+    gboolean set;
+  } av, spu;
 
   GList  *options; /* options are simply script engine command strings */
   play_item_type type;
@@ -51,7 +53,8 @@ struct play_item_s {
   gboolean played; /* this has been played in playlist */
 };
 
-play_item_t *play_item_new (const char *title, const char *mrl, int start_time);
+play_item_t *play_item_new (const char *title, const char *mrl,
+			    int start_time, int duration);
 void play_item_dispose (play_item_t *play_item);
 
 void play_item_add_option (play_item_t *item, const char *option);
@@ -61,7 +64,8 @@ play_item_t *play_item_load (xml_node_t *xplay_item);
 
 play_item_t *play_item_copy (play_item_t *item);
 
-void play_item_play (play_item_t *play_item, int pos, int pos_time);
+void play_item_play (play_item_t *play_item,
+		     int pos, int pos_time, int duration);
 
 /* gui part */
 
