@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2006 the xine-project
+ * Copyright (C) 2004-2007 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,8 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: player.c,v 1.86 2006/04/08 21:34:50 dsalt Exp $
- *
  * player object
  */
 
@@ -57,18 +55,100 @@ xine_video_port_t *video_port;
  * always start xine engine from a seperate thread so the gui stays responsive
  */
 
-static int             goto_pos, goto_time;
+static int             goto_pos, goto_time, end_time;
 static pthread_t       play_thread;
 static char           *cur_mrl = NULL, *cur_title = NULL;
 
+static gboolean demuxer_for_logo = TRUE;
+
+static int play_exec_error_suppress = 0;
+
+void play_exec_error_suppress_next (void)
+{
+  ++play_exec_error_suppress;
+}
+
+static void play_exec_error (void)
+{
+  if (play_exec_error_suppress)
+  {
+#ifdef XINE_PARAM_GAPLESS_SWITCH
+    xine_set_param (stream, XINE_PARAM_GAPLESS_SWITCH, 0);
+#endif
+    --play_exec_error_suppress;
+    return;
+  }
+
+  int error;
+  switch (error = xine_get_error (stream))
+  {
+  case XINE_ERROR_NO_INPUT_PLUGIN:
+    display_error (FROM_XINE_COMBINABLE, _("The xine engine failed to start."),
+		   _("No input plugin was found.\n"
+		     "Maybe the file does not exist or cannot be accessed, "
+		     "or there is an error in the URL."));
+    break;
+
+  case XINE_ERROR_NO_DEMUX_PLUGIN:
+    if (playlist_showing_logo ())
+    {
+      g_idle_add ((GSourceFunc)infobar_show_metadata, infobars);
+      if (demuxer_for_logo)
+      {
+	demuxer_for_logo = FALSE;
+	display_error (FROM_XINE,
+		       _("Whoops. You seem to have a broken xine-lib."),
+		       _("No demuxer found - stream format not recognised."));
+      }
+    }
+    else
+      display_error (FROM_XINE, _("The xine engine failed to start."),
+		     _("No demuxer found - stream format not recognised."));
+    break;
+
+  case XINE_ERROR_DEMUX_FAILED:
+    display_error (FROM_XINE, _("The xine engine failed to start."),
+		   _("Demuxing failed.\n"
+		     "Maybe the stream is corrupt or of an unexpected type."));
+    break;
+
+  case XINE_ERROR_MALFORMED_MRL:
+    display_error (FROM_XINE, _("The xine engine failed to start."),
+		   _("The MRL is malformed."));
+    break;
+
+  case XINE_ERROR_INPUT_FAILED:
+    display_error (FROM_XINE_COMBINABLE, _("The xine engine failed to start."),
+		   _("The stream could not be opened."));
+    break;
+
+
+  default:
+    display_error (FROM_XINE, _("The xine engine failed to start."),
+		   _("Unknown error (code %d)"), error);
+  }
+
+#ifdef XINE_PARAM_GAPLESS_SWITCH
+  xine_set_param (stream, XINE_PARAM_GAPLESS_SWITCH, 0);
+#endif
+
+  if (!playlist_showing_logo ())
+    g_idle_add ((GSourceFunc)playlist_logo, NULL);
+}
+
 static __attribute__ ((noreturn)) void *play_exec (void *data)
 {
   xine_cfg_entry_t entry;
   char *hash, *mrl = NULL;
 
+  pthread_setcancelstate (PTHREAD_CANCEL_ENABLE, NULL);
+  pthread_setcanceltype (PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
+
   /*pthread_mutex_lock (&engine_lock);*/
   pthread_mutex_lock (&widgets_update_lock);
+  pthread_cleanup_push (pthread_mutex_unlock, &widgets_update_lock);
   gdk_threads_enter();
+  pthread_cleanup_push (gdk_threads_leave, NULL);
 
   gxineinfo_clear (infobars);
   gxineinfo_update_line (infobars, 0, "%s", _("opening..."));
@@ -126,93 +206,61 @@ static __attribute__ ((noreturn)) void *play_exec (void *data)
       if (i < G_N_ELEMENTS (subs))
       {
 	char *tmp = g_filename_to_uri (file, NULL, NULL);
-	char *uri = g_filename_to_uri (tmp + 7, NULL, NULL);
-	mrl = g_strconcat (cur_mrl, hash ? ";subtitle:" : "#subtitle:", uri + 7, NULL);
-	free (uri);
+	mrl = g_strconcat (cur_mrl, hash ? ";subtitle:" : "#subtitle:", tmp + 7, NULL);
 	free (tmp);
       }
     }
   }
 
-  gdk_threads_leave ();
-  if (!xine_open (stream, mrl ? : cur_mrl))
+  pthread_cleanup_pop (TRUE);
+  int ret = xine_open (stream, mrl ? : cur_mrl);
+
+  if (ret)
   {
-    int error;
     gdk_threads_enter ();
-    switch (error = xine_get_error (stream))
-    {
-    case XINE_ERROR_NO_INPUT_PLUGIN:
-      display_error (FROM_XINE, _("The xine engine failed to start."),
-		     _("No input plugin was found.\n"
-		     "Maybe the file does not exist or cannot be accessed, "
-		     "or there is an error in the URL."));
-      break;
+    pthread_cleanup_push (gdk_threads_leave, NULL);
+    ui_set_status (playlist_showing_logo () ? UI_STOP : UI_PLAY);
+    infobar_show_metadata (infobars);
+    pthread_cleanup_pop (TRUE);
 
-    case XINE_ERROR_NO_DEMUX_PLUGIN:
-      display_error (FROM_XINE,
-		     playlist_showing_logo ()
-		       ? _("Whoops. You seem to have a broken xine-lib.")
-		       : _("The xine engine failed to start."),
-		     _("No demuxer found - stream format not recognised."));
-      break;
+//    xine_play (stream, 0, 0);
+    xine_play (stream, goto_pos, goto_time);
+    gtk_video_set_end_time ((GtkVideo *)gtv, end_time);
+    goto_pos = 0;
+    goto_time = 0;
+    end_time = 0;
 
-    case XINE_ERROR_DEMUX_FAILED:
-      display_error (FROM_XINE, _("The xine engine failed to start."),
-		     _("Demuxing failed.\n"
-		       "Maybe the stream is corrupt or of an unexpected type."));
-      break;
-
-    case XINE_ERROR_MALFORMED_MRL:
-      display_error (FROM_XINE, _("The xine engine failed to start."),
-		     _("The MRL is malformed."));
-      break;
-
-    case XINE_ERROR_INPUT_FAILED:
-      display_error (FROM_XINE, _("The xine engine failed to start."),
-		     _("The stream could not be opened."));
-      break;
-
-
-    default:
-      display_error (FROM_XINE, _("The xine engine failed to start."),
-		     _("Unknown error (code %d)"), error);
-    }
-
-    if (!playlist_showing_logo ())
-      g_idle_add ((GSourceFunc)playlist_logo, NULL);
-    goto done; /* will call gdk_threads_leave */
+    /* correct for possible stream changes */
+    gdk_threads_enter ();
+    pthread_cleanup_push (gdk_threads_leave, NULL);
+    ui_set_status (playlist_showing_logo () ? UI_STOP : UI_PLAY);
+    infobar_show_metadata (infobars);
+    playlist_check_set_title ();
+    pthread_cleanup_pop (TRUE);
+  }
+  else
+  {
+    gdk_threads_enter ();
+    pthread_cleanup_push (gdk_threads_leave, NULL);
+    play_exec_error ();
+    pthread_cleanup_pop (TRUE);
   }
 
-  gdk_threads_enter ();
-  ui_set_status (playlist_showing_logo () ? UI_STOP : UI_PLAY);
-  infobar_show_metadata (infobars);
-  gdk_threads_leave ();
-
-  xine_play (stream, goto_pos, goto_time);
-  goto_pos = 0;
-  goto_time = 0;
-
-  /* correct for possible stream changes */
-  gdk_threads_enter ();
-  ui_set_status (playlist_showing_logo () ? UI_STOP : UI_PLAY);
-  infobar_show_metadata (infobars);
-  playlist_check_set_title ();
-
-done:
   free (mrl);
-  gdk_threads_leave ();
-  pthread_mutex_unlock (&widgets_update_lock);
+  pthread_cleanup_pop (TRUE);
   /*pthread_mutex_unlock (&engine_lock);*/
   pthread_exit (NULL);
 }
 
-void player_launch (const char *title, const char *mrl, int pos, int pos_time)
+void player_launch (const char *title, const char *mrl,
+		    int pos, int pos_time, int duration)
 {
   int err;
   pthread_attr_t attr;
 
   goto_pos  = pos < 0 ? 0 : pos;
   goto_time = pos_time < 0 ? 0 : pos_time;
+  end_time = duration <= 0 ? 0: pos_time + duration;
   free (cur_title);
   free (cur_mrl);
   cur_title = title && !playlist_showing_logo () ? strdup (title) : NULL;
@@ -220,6 +268,9 @@ void player_launch (const char *title, const char *mrl, int pos, int pos_time)
 
   gtk_video_unblank_screen ((GtkVideo *)gtv);
 
+  if (!demuxer_for_logo && playlist_showing_logo ())
+    return;
+
   /* start separate thread, so xine_open will not block ui */
   pthread_attr_init (&attr);
   pthread_attr_setstacksize (&attr, 256 * 1024); /* probably overkill */
@@ -241,6 +292,12 @@ void player_wait (void)
   pthread_join (play_thread, NULL);
 }
 
+void player_stop (void)
+{
+  pthread_cancel (play_thread);
+  xine_stop (stream);
+}
+
 static void set_mute (int mute)
 {
   ui_set_status (mute ? UI_AUDIO_MUTE : UI_AUDIO_UNMUTE);
@@ -291,33 +348,19 @@ static void set_zoom (int zoom)
   xine_set_param (stream, XINE_PARAM_VO_ZOOM_Y, zoom);
 }
 
-#define ACTION_SET(Func) \
-  static void set_##Func (int v) \
-  { \
-    if (gtk_toggle_action_get_active (action_items.Func) != v) \
-      gtk_action_activate (GTK_ACTION(action_items.Func)); \
-  }
-
-ACTION_SET(fullscreen)
-static void get_fullscreen (se_prop_t *prop, se_prop_read_t *value)
-{
-  value->i = gtk_video_is_fullscreen ((GtkVideo *)gtv);
-}
-
-ACTION_SET(deinterlace)
-static void get_deinterlace (se_prop_t *prop, se_prop_read_t *value)
-{
-  value->i = gtk_video_get_use_post_plugins_deinterlace ((GtkVideo *)gtv);
-}
-
 static void set_subtitles (int sub)
 {
   if (sub >= -2 && sub <= GXINE_MAX_SPU_CHANNEL)
   {
     GSList *menuitem = action_items.subtitles;
-    int count = g_slist_length (menuitem) - 3;
-    if (count >= sub)
+    int count = g_slist_length (menuitem);
+    /* last three menu items (in definition order, not menu display order):
+     * None (-2), Auto (-1), Other (>= 16, hidden)
+     */
+    if (count > sub + 3)
     {
+      if (sub < 0)
+        count = 1; /* for None, Auto */
       while (menuitem && --count > sub)
 	menuitem = menuitem->next;
       if (menuitem)
@@ -327,7 +370,7 @@ static void set_subtitles (int sub)
       }
     }
     /* not in menu */
-    gtk_action_activate (g_slist_last (action_items.subtitles)->data);
+    gtk_action_activate (action_items.subtitles->data);
     xine_set_param (stream, XINE_PARAM_SPU_CHANNEL, sub);
   }
 }
@@ -346,36 +389,6 @@ static void set_aspect (int aspect)
     gtk_action_activate (menuitem->data);
 }
 
-ACTION_SET(auto_resize)
-static void get_auto_resize (se_prop_t *prop, se_prop_read_t *value)
-{
-  value->i = gtk_video_get_auto_resize ((GtkVideo *)gtv);
-}
-
-ACTION_SET(auto_rescale)
-static void get_auto_rescale (se_prop_t *prop, se_prop_read_t *value)
-{
-  value->i = gtk_video_get_auto_rescale ((GtkVideo *)gtv);
-}
-
-ACTION_SET(vo_postproc)
-static void get_vo_postproc (se_prop_t *prop, se_prop_read_t *value)
-{
-  value->i = gtk_video_get_use_post_plugins_video ((GtkVideo *)gtv);
-}
-
-ACTION_SET(ao_postproc)
-static void get_ao_postproc (se_prop_t *prop, se_prop_read_t *value)
-{
-  value->i = gtk_video_get_use_post_plugins_audio ((GtkVideo *)gtv);
-}
-
-ACTION_SET(wm_unblank)
-static void get_wm_unblank (se_prop_t *prop, se_prop_read_t *value)
-{
-  value->i = gtk_video_get_windowed_unblank ((GtkVideo *)gtv);
-}
-
 static void update_postproc_chain (const char *pref, const char *value)
 {
   xine_cfg_entry_t entry;
@@ -404,6 +417,13 @@ static JSBool set_vo_postproc_chain (JSContext *cx, JSObject *obj,
 			     cx, obj, argc, argv, rval);
 }
 
+static JSBool set_deint_postproc_chain (JSContext *cx, JSObject *obj,
+					uintN argc, jsval *argv, jsval *rval)
+{
+  return set_postproc_chain ("vo_deinterlace.set_chain", "gui.post_plugins.deinterlace",
+			     cx, obj, argc, argv, rval);
+}
+
 static JSBool set_ao_postproc_chain (JSContext *cx, JSObject *obj,
 				     uintN argc, jsval *argv, jsval *rval)
 {
@@ -428,13 +448,35 @@ static JSBool get_time (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
-#ifdef WITH_DEPRECATED
+static JSBool get_pos (JSContext *cx, JSObject *obj, uintN argc,
+		       jsval *argv, jsval *rval)
+{
+  int pos, pos_time, len;
+  se_log_fncall ("get_pos");
+  xine_get_pos_length (stream, &pos, &pos_time, &len);
+
+  jsdouble *pos_percent = JS_NewDouble (cx, pos / 655.35);
+  *rval = DOUBLE_TO_JSVAL (pos_percent);
+  return JS_TRUE;
+}
+
+static JSBool has_time (JSContext *cx, JSObject *obj, uintN argc,
+			jsval *argv, jsval *rval)
+{
+  int pos, pos_time, len;
+  se_log_fncall ("has_time");
+  xine_get_pos_length (stream, &pos, &pos_time, &len);
+  *rval = BOOLEAN_TO_JSVAL (len != 0);
+  return JS_TRUE;
+}
+
+#ifdef WITH_OBSOLETE
 
 static JSBool js_get_speed (JSContext *cx, JSObject *obj, uintN argc,
 			    jsval *argv, jsval *rval)
 {
   se_prop_read_t speed;
-  se_log_fncall_deprecated ("get_speed");
+  se_log_fncall_obsolete ("get_speed");
   get_speed (NULL, &speed);
   *rval = INT_TO_JSVAL (speed.i);
   return JS_TRUE;
@@ -445,7 +487,7 @@ static JSBool js_get_volume (JSContext *cx, JSObject *obj, uintN argc,
 {
   /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
   int   volume;
-  se_log_fncall_deprecated ("get_volume");
+  se_log_fncall_obsolete ("get_volume");
   volume = xine_get_param (stream, XINE_PARAM_AUDIO_VOLUME);
   *rval = INT_TO_JSVAL (volume);
   return JS_TRUE;
@@ -456,7 +498,7 @@ static JSBool js_get_mute (JSContext *cx, JSObject *obj, uintN argc,
 {
   /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
   int mute;
-  se_log_fncall_deprecated ("get_mute");
+  se_log_fncall_obsolete ("get_mute");
   mute = xine_get_param (stream, XINE_PARAM_AUDIO_MUTE);
   *rval = BOOLEAN_TO_JSVAL (mute);
   return JS_TRUE;
@@ -467,13 +509,13 @@ static JSBool js_get_zoom (JSContext *cx, JSObject *obj, uintN argc,
 {
   /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
   int   zoom;
-  se_log_fncall_deprecated ("get_zoom");
+  se_log_fncall_obsolete ("get_zoom");
   zoom = xine_get_param (stream, XINE_PARAM_VO_ZOOM_X);
   *rval = INT_TO_JSVAL (zoom);
   return JS_TRUE;
 }
 
-#endif /* WITH_DEPRECATED */
+#endif /* WITH_OBSOLETE */
 
 static JSBool controls_pause (JSContext *cx, JSObject *obj, uintN argc,
 			      jsval *argv, jsval *rval)
@@ -511,7 +553,7 @@ static JSBool controls_stop (JSContext *cx, JSObject *obj, uintN argc,
 			     jsval *argv, jsval *rval)
 {
   se_log_fncall_checkinit ("stop");
-  xine_stop (stream);
+  player_stop ();
   playlist_logo (NULL);
   ui_set_control_adjustment (Control_SEEKER, 0);
   return JS_TRUE;
@@ -522,7 +564,7 @@ static JSBool controls_eject (JSContext *cx, JSObject *obj, uintN argc,
 {
   se_log_fncall_checkinit ("eject");
   if (xine_get_status (stream) == XINE_STATUS_PLAY)
-    xine_stop (stream);
+    player_stop ();
   gdk_threads_leave (); /* need to do this to allow play_exec() to proceed */
   playlist_logo (cx); /* doesn't matter so long as it's not NULL */
   gdk_threads_enter ();
@@ -532,14 +574,14 @@ static JSBool controls_eject (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
-#ifdef WITH_DEPRECATED
+#ifdef WITH_OBSOLETE
 
 static JSBool js_set_speed (JSContext *cx, JSObject *obj, uintN argc,
 			    jsval *argv, jsval *rval)
 {
   int32 speed;
 
-  se_log_fncall_deprecated ("set_speed");
+  se_log_fncall_obsolete ("set_speed");
 
   se_argc_check (1, "set_speed");
   se_arg_is_int (0, "set_speed");
@@ -558,7 +600,7 @@ static JSBool js_set_volume (JSContext *cx, JSObject *obj, uintN argc,
 {
   int32 volume;
 
-  se_log_fncall_deprecated ("set_volume");
+  se_log_fncall_obsolete ("set_volume");
   se_argc_check (1, "set_volume");
   se_arg_is_int (0, "set_volume");
 
@@ -577,7 +619,7 @@ static JSBool js_set_mute (JSContext *cx, JSObject *obj, uintN argc,
 {
   JSBool mute;
 
-  se_log_fncall_deprecated ("set_mute");
+  se_log_fncall_obsolete ("set_mute");
   se_argc_check_max (1, "set_mute");
 
   if (argc == 1)
@@ -598,7 +640,7 @@ static JSBool js_set_zoom (JSContext *cx, JSObject *obj, uintN argc,
 {
   int32 zoom;
 
-  se_log_fncall_deprecated ("set_zoom");
+  se_log_fncall_obsolete ("set_zoom");
   se_argc_check (1, "set_zoom");
   se_arg_is_int (0, "set_zoom");
 
@@ -608,7 +650,7 @@ static JSBool js_set_zoom (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
-#endif /* WITH_DEPRECATED */
+#endif /* WITH_OBSOLETE */
 
 static int zoomcb, zoomcb_id = 0;
 
@@ -676,16 +718,20 @@ static JSBool js_set_fullscreen (JSContext *cx, JSObject *obj, uintN argc,
   else
     fs = !gtk_video_is_fullscreen ((GtkVideo *)gtv);
 
-  set_fullscreen (fs);
+  if (gtk_toggle_action_get_active (action_items.fullscreen) != fs)
+    gtk_action_activate (GTK_ACTION (action_items.fullscreen));
+
   return JS_TRUE;
 }
 
+#ifdef WITH_OBSOLETE
+
 static JSBool js_set_deinterlace (JSContext *cx, JSObject *obj, uintN argc,
 				 jsval *argv, jsval *rval)
 {
   int32 di;
 
-  se_log_fncall_deprecated ("set_deinterlace");
+  se_log_fncall_obsolete ("set_deinterlace");
   se_argc_check_max (1, "set_deinterlace");
 
   if (argc == 1)
@@ -706,7 +752,7 @@ static JSBool js_set_postproc_video (JSContext *cx, JSObject *obj,
   se_t *se = (se_t *) JS_GetContextPrivate(cx);
   int32 di;
 
-  se_log_fncall_deprecated ("set_postproc_video");
+  se_log_fncall_obsolete ("set_postproc_video");
   se_argc_check_max (1, "set_postproc_video");
 
   if (argc == 1)
@@ -735,7 +781,7 @@ static JSBool js_set_postproc_audio (JSContext *cx, JSObject *obj,
   se_t *se = (se_t *) JS_GetContextPrivate(cx);
   int32 di;
 
-  se_log_fncall_deprecated ("set_postproc_audio");
+  se_log_fncall_obsolete ("set_postproc_audio");
   se_argc_check_max (1, "set_postproc_audio");
 
   if (argc == 1)
@@ -763,7 +809,7 @@ static JSBool js_set_auto_resize (JSContext *cx, JSObject *obj, uintN argc,
 {
   int32 ar;
 
-  se_log_fncall_deprecated ("set_auto_resize");
+  se_log_fncall_obsolete ("set_auto_resize");
   se_argc_check_max (1, "set_auto_resize");
 
   if (argc == 1)
@@ -783,7 +829,7 @@ static JSBool js_set_auto_rescale (JSContext *cx, JSObject *obj, uintN argc,
 {
   int32 ar;
 
-  se_log_fncall_deprecated ("set_auto_rescale");
+  se_log_fncall_obsolete ("set_auto_rescale");
   se_argc_check_max (1, "set_auto_rescale");
 
   if (argc == 1)
@@ -803,7 +849,7 @@ static JSBool js_set_aspect (JSContext *cx, JSObject *obj, uintN argc,
 {
   int32 aspect;
 
-  se_log_fncall_deprecated ("set_aspect");
+  se_log_fncall_obsolete ("set_aspect");
   se_argc_check_max (1, "set_aspect");
 
   if (argc==1)
@@ -818,6 +864,7 @@ static JSBool js_set_aspect (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
+#endif /* WITH_OBSOLETE */
 #endif /* WITH_DEPRECATED */
 
 static JSBool js_snapshot (JSContext *cx, JSObject *obj, uintN argc,
@@ -878,10 +925,12 @@ static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc,
 
   case 2:
     {
-      se_arg_is_int (0, "play");
+      se_arg_is_number (0, "play");
       se_arg_is_int (1, "play");
 
-      JS_ValueToInt32 (cx, argv[0], &pos);
+      jsdouble *posv = JS_NewDouble (cx, 0); /* when is this freed? */
+      JS_ValueToNumber (cx, argv[0], posv);
+      pos = (*posv >= 0 && *posv < 100) ? *posv * 655.35 : -1;
       JS_ValueToInt32 (cx, argv[1], &pos_time);
       mrl = NULL;
       se_log ("playing from %d, %d\n", pos, pos_time);
@@ -890,13 +939,14 @@ static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc,
 
   case 3:
     {
-      JSString *str;
       se_arg_is_string (0, "play");
-      se_arg_is_int (1, "play");
+      se_arg_is_number (1, "play");
       se_arg_is_int (2, "play");
 
-      str = JS_ValueToString (cx, argv[0]);
-      JS_ValueToInt32 (cx, argv[1], &pos);
+      JSString *str = JS_ValueToString (cx, argv[0]);
+      jsdouble *posv = JS_NewDouble (cx, 0); /* when is this freed? */
+      JS_ValueToNumber (cx, argv[1], posv);
+      pos = (*posv >= 0 && *posv < 100) ? *posv * 655.35 : -1;
       JS_ValueToInt32 (cx, argv[2], &pos_time);
       mrl = JS_GetStringBytes (str);
       se_log ("playing '%s' from %d, %d\n", mrl, pos, pos_time);
@@ -904,9 +954,6 @@ static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc,
     break;
   }
 
-  if (pos > 0)
-    pos *= 655;
-
   if (mrl)
     playlist_play_from (playlist_add_mrl (mrl, -1), pos, pos_time);
   else
@@ -915,7 +962,7 @@ static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc,
 
     if (item)
     {
-      switch (xine_get_status (stream))
+      switch (playlist_showing_logo () ? XINE_STATUS_IDLE : xine_get_status (stream))
       {
       case XINE_STATUS_STOP:
       case XINE_STATUS_IDLE:
@@ -934,9 +981,7 @@ static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc,
     } else
       logprintf (_("script_engine: error: no valid play item available\n"));
   }
-
-  if (xine_get_param (stream, XINE_PARAM_SPEED) != XINE_SPEED_NORMAL)
-    set_speed (XINE_SPEED_NORMAL);
+  set_speed (XINE_SPEED_NORMAL);
 
   return JS_TRUE;
 }
@@ -1002,17 +1047,14 @@ static xine_audio_port_t *load_audio_out_driver (void)
 {
   xine_audio_port_t      *audio_port;
 
+  char **choices = get_driver_ids (xine_list_audio_output_plugins (xine));
+  /* try to init audio with stored information */
+  int i = xine_config_register_enum (xine,
+				     "audio.driver", 0, choices,
+				     _("audio driver to use"),
+				     NULL, 10, GXINE_TRUE, NULL);
   if (!audio_driver_id)
-  {
-    char **choices = get_driver_ids (xine_list_audio_output_plugins (xine));
-    /* try to init audio with stored information */
-    int i = xine_config_register_enum (xine,
-                                   "audio.driver", 0,
-                                   choices,
-                                   _("audio driver to use"),
-                                   NULL, 10, gtk_true, NULL);
     audio_driver_id = choices[i];
-  }
 
   if (!strcmp (audio_driver_id, "null"))
     return NULL;
@@ -1022,16 +1064,33 @@ static xine_audio_port_t *load_audio_out_driver (void)
     audio_port = xine_open_audio_driver (xine, audio_driver_id, NULL);
     if (audio_port)
       return audio_port;
-    else
-      g_printerr (_("audio driver %s failed\n"), audio_driver_id);
+
+    g_printerr (_("audio driver %s failed\n"), audio_driver_id);
   }
 
   /* autoprobe */
-  return xine_open_audio_driver (xine, NULL, NULL);
+  audio_port = xine_open_audio_driver (xine, NULL, NULL);
+  if (audio_port)
+    return audio_port;
+
+  g_printerr (_("audio driver %s failed\n"), "auto");
+  return xine_open_audio_driver (xine, "none", NULL);
 }
 
 /* Properties */
 
+static void add_chain_funcs (const char *prop, const char *config,
+			     JSBool (*set_chain) (JSContext *, JSObject *,
+						  uintN, jsval *, jsval *))
+{
+  se_o_t *obj = se_find_object (gse, NULL, prop);
+  if (set_chain)
+    se_defun (gse, obj, "set_chain", set_chain, 0, 0, SE_GROUP_HIDDEN, NULL, NULL);
+  se_prop_create_xine_id (gse, obj, "chain", config);
+  se_prop_add_listener (gse, obj, "chain",
+			listen_postproc_chain, (void *)config);
+}
+
 void player_init (void)
 {
 /*
@@ -1043,9 +1102,19 @@ void player_init (void)
   /* load drivers, create xine stream */
 
   audio_port = load_audio_out_driver ();
+  if (!audio_port)
+  {
+    display_error_modal (FROM_GXINE, _("Fatal error"),
+			 _("No audio output driver could be loaded."));
+    exit (2);
+  }
+
   video_port = xine_open_video_driver (xine, "none", 0, NULL);
 
   stream = xine_stream_new (xine, audio_port, video_port);
+#ifdef XINE_PARAM_EARLY_FINISHED_EVENT
+  xine_set_param (stream, XINE_PARAM_EARLY_FINISHED_EVENT, 1);
+#endif
 
   /*
    * set up script engine objects and functions
@@ -1065,26 +1134,35 @@ void player_init (void)
        *     VARIABLE[=TYPE] or METHOD([PARAMETER TYPE,...])
        *   To the right, descriptive text
        */
-      { "av_speed", N_("v=int, min, max; playback speed"), 0, get_speed, set_speed, XINE_SPEED_PAUSE, XINE_SPEED_FAST_4, FALSE, { listen_speed } },
-      { "vo_aspect", N_("v=int; aspect ratio"), XINE_PARAM_VO_ASPECT_RATIO, NULL, set_aspect, 0, XINE_VO_ASPECT_NUM_RATIOS - 1, TRUE },
-      { "vo_subtitle", N_("v=int, min, max; subtitle channel"), XINE_PARAM_SPU_CHANNEL, NULL, set_subtitles, -2, GXINE_MAX_SPU_CHANNEL },
-      { "vo_zoom", N_("v=int, min, max; video zoom (within window)"), XINE_PARAM_VO_ZOOM_X, NULL, set_zoom, XINE_VO_ZOOM_MIN, XINE_VO_ZOOM_MAX },
+      { "av_speed", N_("v=int, min, max; playback speed"), 
+       UI_CUSTOM (get_speed, set_speed), 
+       XINE_SPEED_PAUSE, 
+       XINE_SPEED_FAST_4, 
+       FALSE, 
+       { listen_speed } },
+      { "vo_aspect", N_("v=int; aspect ratio"), UI_XINE_PARAM (XINE_PARAM_VO_ASPECT_RATIO, set_aspect), 0, XINE_VO_ASPECT_NUM_RATIOS - 1, TRUE },
+      { "vo_subtitle", N_("v=int, min, max; subtitle channel"), UI_XINE_PARAM (XINE_PARAM_SPU_CHANNEL, set_subtitles), -2, GXINE_MAX_SPU_CHANNEL },
+      { "vo_zoom", N_("v=int, min, max; video zoom (within window)"), UI_XINE_PARAM (XINE_PARAM_VO_ZOOM_X, set_zoom), XINE_VO_ZOOM_MIN, XINE_VO_ZOOM_MAX },
       { NULL }
     };
     static const ui_property_t bool_props[] = {
-      { "ao_mute", N_("v=bool, toggle(): audio mute"), XINE_PARAM_AUDIO_MUTE, NULL, set_mute },
-      { "vo_auto_resize", N_("v=bool, toggle(): auto-resize video on frame shape change"), 0, get_auto_resize, set_auto_resize },
-      { "vo_auto_rescale", N_("v=bool, toggle(): double size of low-resolution video"), 0, get_auto_rescale, set_auto_rescale },
-      { "vo_fullscreen", N_("v=bool, toggle(): full-screen mode"), 0, get_fullscreen, set_fullscreen },
-      { "vo_deinterlace", N_("v=bool, toggle(): enable deinterlacing"), 0, get_deinterlace, set_deinterlace },
-      { "ao_postproc", N_("v=bool, toggle(): audio post-processing; chain=string: plugin chain"), 0, get_ao_postproc, set_ao_postproc },
-      { "vo_postproc", N_("v=bool, toggle(): video post-processing; chain=string: plugin chain"), 0, get_vo_postproc, set_vo_postproc },
-      { "wm_unblank", N_("v=bool, toggle(): unblanking in windowed mode"), 0, get_wm_unblank, set_wm_unblank },
+      { "ao_mute", N_("v=bool, toggle(): audio mute"), UI_XINE_PARAM (XINE_PARAM_AUDIO_MUTE, set_mute) },
+      { "vo_auto_resize", N_("v=bool, toggle(): auto-resize video on frame shape change"), UI_GTK_VIDEO (auto_resize, gtk_video_get_auto_resize) },
+      { "vo_auto_rescale", N_("v=bool, toggle(): double size of low-resolution video"), UI_GTK_VIDEO (auto_rescale, gtk_video_get_auto_rescale) },
+      { "vo_fullscreen", N_("v=bool, toggle(): full-screen mode"), UI_GTK_VIDEO (fullscreen, gtk_video_is_fullscreen) },
+      { "vo_deinterlace", N_("v=bool, toggle(): enable deinterlacing"), UI_GTK_VIDEO (deinterlace, gtk_video_get_use_post_plugins_deinterlace) },
+      { "ao_postproc", N_("v=bool, toggle(): audio post-processing; chain=string: plugin chain"), UI_GTK_VIDEO (ao_postproc, gtk_video_get_use_post_plugins_audio) },
+      { "vo_postproc", N_("v=bool, toggle(): video post-processing; chain=string: plugin chain"), UI_GTK_VIDEO (vo_postproc, gtk_video_get_use_post_plugins_video) },
+      { "wm_unblank", N_("v=bool, toggle(): unblanking in windowed mode"), UI_GTK_VIDEO (wm_unblank, gtk_video_get_windowed_unblank) },
       { NULL }
     };
     static const ui_property_t zoom_subprops[] = {
-      { "x", N_("v=int, min, max; video zoom (within window)"), XINE_PARAM_VO_ZOOM_X, NULL, NULL, XINE_VO_ZOOM_MIN, XINE_VO_ZOOM_MAX },
-      { "y", N_("v=int, min, max; video zoom (within window)"), XINE_PARAM_VO_ZOOM_Y, NULL, NULL, XINE_VO_ZOOM_MIN, XINE_VO_ZOOM_MAX },
+      { "x", N_("v=int, min, max; video zoom (within window)"), UI_XINE_PARAM (XINE_PARAM_VO_ZOOM_X, NULL), XINE_VO_ZOOM_MIN, XINE_VO_ZOOM_MAX },
+      { "y", N_("v=int, min, max; video zoom (within window)"), UI_XINE_PARAM (XINE_PARAM_VO_ZOOM_Y, NULL), XINE_VO_ZOOM_MIN, XINE_VO_ZOOM_MAX },
+      { NULL }
+    };
+    static const ui_property_t fs_subprops[] = {
+      { "sticky", N_("v=bool, toggle(); forced stickiness in full-screen mode"), UI_BOOL (fs_sticky, fs_always_sticky, NULL) },
       { NULL }
     };
 
@@ -1092,22 +1170,14 @@ void player_init (void)
     ui_create_properties (bool_props, NULL, SE_TYPE_BOOL);
     ui_create_properties (zoom_subprops, se_find_object (gse, NULL, "vo_zoom"),
 			  SE_TYPE_INT);
+    ui_create_properties (fs_subprops, se_find_object (gse, NULL, "vo_fullscreen"),
+			  SE_TYPE_BOOL);
   }
 
   /* a few extras - chain definition should be done via the GUI */
-  se_o_t *obj = se_find_object (gse, NULL, "ao_postproc");
-  se_defun (gse, obj, "set_chain",
-	    set_ao_postproc_chain, 0, 0, SE_GROUP_HIDDEN, NULL, NULL);
-  se_prop_create_xine_id (gse, obj, "chain", "gui.post_plugins.audio");
-  se_prop_add_listener (gse, obj, "chain",
-			listen_postproc_chain, "gui.post_plugins.audio");
-
-  obj = se_find_object (gse, NULL, "vo_postproc");
-  se_defun (gse, obj, "set_chain",
-	    set_vo_postproc_chain, 0, 0, SE_GROUP_HIDDEN, NULL, NULL);
-  se_prop_create_xine_id (gse, obj, "chain", "gui.post_plugins.video");
-  se_prop_add_listener (gse, obj, "chain",
-			listen_postproc_chain, "gui.post_plugins.video");
+  add_chain_funcs ("ao_postproc", "gui.post_plugins.audio", set_ao_postproc_chain);
+  add_chain_funcs ("vo_postproc", "gui.post_plugins.video", set_vo_postproc_chain);
+  add_chain_funcs ("vo_deinterlace", "gui.post_plugins.deinterlace", set_deint_postproc_chain);
 
   /*
    * hook up global xine functions for convenience
@@ -1120,7 +1190,11 @@ void player_init (void)
 	SE_GROUP_ENGINE, N_("[mrl] [, pos, time]"),
 	N_("time in milliseconds") },
       { "get_time", get_time, 0, 0,
-	SE_GROUP_ENGINE, NULL, NULL },
+	SE_GROUP_ENGINE, NULL, N_("stream time index") },
+      { "get_pos", get_pos, 0, 0,
+	SE_GROUP_ENGINE, NULL, N_("stream position (%)") },
+      { "has_time", has_time, 0, 0,
+	SE_GROUP_ENGINE, NULL, N_("whether the stream has a time index") },
       { "pause", controls_pause, 0, 0,
   	SE_GROUP_ENGINE, N_("[bool]"), NULL },
       { "stop", controls_stop, 0, 0,
@@ -1137,6 +1211,7 @@ void player_init (void)
 	SE_GROUP_PROPERTIES, N_("int"), NULL },
 
 #ifdef WITH_DEPRECATED
+#ifdef WITH_OBSOLETE
       { "set_speed", js_set_speed, 0, 0,
 	SE_GROUP_HIDDEN, NULL, NULL },
       { "get_speed", js_get_speed, 0, 0,
@@ -1153,8 +1228,10 @@ void player_init (void)
 	SE_GROUP_HIDDEN, NULL, NULL },
       { "get_zoom", js_get_zoom, 0, 0,
 	SE_GROUP_HIDDEN, NULL, NULL },
+#endif /* WITH_OBSOLETE */
       { "set_fullscreen", js_set_fullscreen, 0, 0,
 	SE_GROUP_HIDDEN, NULL, NULL },
+#ifdef WITH_OBSOLETE
       { "set_aspect", js_set_aspect, 0, 0,
 	SE_GROUP_HIDDEN, NULL, NULL },
       { "set_deinterlace", js_set_deinterlace, 0, 0,
@@ -1167,6 +1244,7 @@ void player_init (void)
 	SE_GROUP_HIDDEN, NULL, NULL },
       { "set_auto_rescale", js_set_auto_rescale, 0, 0,
 	SE_GROUP_HIDDEN, NULL, NULL },
+#endif /* WITH_OBSOLETE */
 #endif /* WITH_DEPRECATED */
 
       { "is_live_stream", js_is_live_stream, 0, 0,
@@ -1186,15 +1264,21 @@ void player_init (void)
        _("The main window's default size"),
        NULL, 0, NULL, NULL);
     xine_config_register_bool
-      (xine, "gui.magnify_lowres_video", 0,
+      (xine, "gui.windowed_mode.magnify_lowres_video", CONFIG_VERSION_ATLEAST (0, 6, 0),
        _("Double the size of small video streams"),
        _("This affects video streams whose dimensions are at most ⅓ "
 	 "of those of the display."),
-       0, gtk_true, NULL);
+       0, GXINE_TRUE, NULL);       
+    config_update_default ("gui.windowed_mode.magnify_lowres_video", 1);
+    xine_config_register_bool
+      (xine, "gui.windowed_mode.auto_resize", 1,
+       _("Automatic video window resizing"),
+       _("Whether the window is automatically resized on a stream format change."),
+       0, NULL, NULL);
     xine_config_register_bool
       (xine, "gui.subtitle_autoload", 1,
        _("Subtitle autoloading"),
        _("Automatically load subtitles if they exist."),
-       0, gtk_true, NULL);
+       0, GXINE_TRUE, NULL);
   }
 }
