@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2006 the xine project
+ * Copyright (C) 2001-2007 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,8 +15,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: preferences.c,v 1.61 2006/04/05 19:08:33 dsalt Exp $
- *
  * Functions to stup and deal with a preferences dialog interfacing with
  * the xine config system.
  *
@@ -41,6 +39,7 @@
 
 #include "mediamarks.h"
 #include "preferences.h"
+#include "ui.h"
 #include "utils.h"
 
 #include <jsapi.h>
@@ -48,7 +47,7 @@
 
 typedef struct {
   void *notebook; /* NULL - get from parent */
-  GtkWidget *editable, *label, *separator;
+  GtkWidget *todefault, *revert, *editable, *label, *separator;
   int exp;
 } pref_item_t;
 #define PREF_ITEM_T(NODE) ((pref_item_t *)(NODE)->data)
@@ -117,6 +116,8 @@ static int select_show_prefs_internal (GNode *page, int exp)
       }
       if (pref->exp <= exp)
 	last = pref;
+      func (pref->todefault);
+      func (pref->revert);
       func (pref->editable);
       func (pref->label);
       func (pref->separator);
@@ -289,10 +290,119 @@ static void enum_cb (GtkWidget* widget, gpointer data)
   }
 }
 
+#ifdef WITH_HAL
+#include <unistd.h>
+#include <hal/libhal.h>
+
+static LibHalContext *hal = NULL;
+
+static gboolean check_hal (void)
+{
+  if (hal)
+    return TRUE;
+  DBusConnection *dbus = dbus_bus_get (DBUS_BUS_SYSTEM, NULL);
+  if (!dbus)
+    return FALSE;
+  hal = libhal_ctx_new ();
+  if (!hal)
+    return FALSE;
+  libhal_ctx_set_dbus_connection (hal, dbus);
+  return TRUE;
+}
+
+static char *
+get_hal_property (const char *dev, const char *dev_tag, const char *prop_tag)
+{
+  int count;
+  char **devs = libhal_manager_find_device_string_match
+		  (hal, dev_tag, dev, &count, NULL);
+  if (!devs)
+    return NULL;
+  if (!devs[0])
+  {
+    libhal_free_string_array (devs);
+    return NULL;
+  }
+
+  char *prop = libhal_device_get_property_string (hal, devs[0], prop_tag, NULL);
+  char *ret = (prop && prop[0]) ? strdup (prop) : NULL;
+  libhal_free_string (prop);
+  libhal_free_string_array (devs);
+  return ret;
+}
+
+static char *
+get_hal_device_info (const char *dev)
+{
+  if (!check_hal () || !dev)
+    return NULL;
+
+  int ldev = strlen (dev);
+  char buf[FILENAME_MAX + ldev + 1];
+  strcpy (buf, dev); /* safe! */
+  if (buf[0] == '/')
+  {
+    ldev = strrchr (buf, '/') - buf + 1;
+    int tmp = readlink (dev, buf + ldev, FILENAME_MAX - 1);
+    if (tmp <= 0)
+      strcpy (buf, dev); /* safe! */
+    else if (buf[ldev] == '/')
+    {
+      memmove (buf, buf + ldev, tmp);
+      buf[tmp] = 0;
+    }
+    else
+    {
+      char *p;
+      buf[ldev + tmp] = 0;
+      while ((p = strstr (buf, "//")))
+        memmove (p, p + 1, strlen (p));
+      while ((p = strstr (buf, "/./")))
+        memmove (p, p + 2, strlen (p) - 1);
+      while ((p = strstr (buf, "/../")))
+      {
+        char *q = p;
+        while (--q >= buf && *q != '/')
+          /**/;
+        memmove (q, p + 3, strlen (p) - 2);
+      }
+    }
+  }
+
+#ifdef HAL_DEVICE_FILE_PREFIX
+  char *const buf_unprefixed =
+    strncmp (buf, HAL_DEVICE_FILE_PREFIX, sizeof (HAL_DEVICE_FILE_PREFIX) - 1)
+    ? "" : buf + sizeof (HAL_DEVICE_FILE_PREFIX) - 2;
+#else
+  char *const buf_unprefixed = buf;
+#endif
+
+  return get_hal_property (buf, "block.device", "storage.model")
+	 ? : get_hal_property (buf, "block.device", "info.product")
+	 ? : get_hal_property (buf_unprefixed, HAL_DEVICE_FILE_PROPERTY, "info.product")
+	 /* FIXME - BSD etc. */
+	 ;
+}
+
+static void
+display_hal_device_info (GtkWidget *widget, const char *dev)
+{
+  GtkLabel *info = g_object_get_data (G_OBJECT (widget), "extra");
+  if (info)
+  {
+    char *hal = get_hal_device_info (dev);
+    char *content = g_markup_printf_escaped ("<small>%s</small>", hal ? : "");
+    gtk_label_set_markup (info, content);
+    free (content);
+    free (hal);
+  }
+}
+#endif /* WITH_HAL */
+
 static void file_cb (GtkWidget *widget, int response, gpointer data)
 {
   xine_cfg_entry_t entry;
-  gchar *key = (gchar *) data;
+  gchar *key = g_object_get_data (data, "cfg");
 
   if (update_lock || response != GTK_RESPONSE_ACCEPT)
     return;
@@ -308,6 +418,10 @@ static void file_cb (GtkWidget *widget, int response, gpointer data)
     logprintf ("preferences: updating entry\n");
     entry.str_value = key;
     xine_config_update_entry (xine, &entry);
+#ifdef WITH_HAL
+    if (entry.num_value == 2 /* XINE_CONFIG_STRING_IS_DEVICE_NAME */)
+      display_hal_device_info (data, key); 
+#endif
   }
   free (key);
 }
@@ -317,14 +431,141 @@ static void file_activate_cb (GtkWidget *widget, gpointer data)
   file_cb (widget, GTK_RESPONSE_ACCEPT, data);
 }
 
-static void file_map_cb (GtkWidget *fcb, gpointer key)
+#ifdef WITH_HAL
+static void file_preview_cb (GtkFileChooser *fc, gpointer data)
 {
-  if (!g_object_get_data (fcb, "gxine"))
+  GtkLabel *info = GTK_LABEL (gtk_file_chooser_get_preview_widget (fc));
+  char *file = gtk_file_chooser_get_preview_filename (fc);
+  if (!file)
+    file = gtk_file_chooser_get_filename (fc);
+  char *content = get_hal_device_info (file);
+  gtk_label_set_text (info, content ? : "");
+  free (content);
+  g_free (file);
+}
+#endif
+
+static void file_map_cb (GtkFileChooser *fcb, gpointer fc)
+{
+  if (!g_object_get_data (fc, "gxine"))
   {
-    g_signal_connect (fcb, "current-folder-changed",
-		      G_CALLBACK(file_activate_cb), key);
-    g_object_set_data (fcb, "gxine", fcb);
+    g_signal_connect_swapped (fcb, "current-folder-changed",
+			      G_CALLBACK(file_activate_cb), fc);
+    g_object_set_data (fc, "gxine", fc);
   }
+#ifdef WITH_HAL
+  file_preview_cb (fcb, fc);
+#endif
+}
+
+static void
+default_item_cb (GtkWidget *w, gpointer key)
+{
+  xine_cfg_entry_t entry;
+  xine_config_lookup_entry (xine, key, &entry);
+  if (entry.type == XINE_CONFIG_TYPE_STRING)
+    entry.str_value = entry.str_default;
+  else
+    entry.num_value = entry.num_default;
+  preferences_update_entry (&entry);
+}
+
+static inline GtkWidget *
+create_item_default (const xine_cfg_entry_t * entry)
+{
+  char *tip;
+  gboolean alloc = FALSE;
+
+  switch (entry->type)
+  {
+  case XINE_CONFIG_TYPE_ENUM:
+    tip = entry->enum_values[entry->num_default];
+    break;
+
+  case XINE_CONFIG_TYPE_RANGE:
+  case XINE_CONFIG_TYPE_NUM:
+    alloc = TRUE;
+    tip = g_strdup_printf ("%d", entry->num_default);
+    break;
+
+  case XINE_CONFIG_TYPE_BOOL:
+    tip = entry->num_default ? "✓" : "✗";
+    break;
+
+  case XINE_CONFIG_TYPE_STRING:
+    tip = entry->str_default;
+    break;
+
+  default:
+    return NULL;
+  }
+
+  GtkWidget *widget = ui_button_new_stock (GTK_STOCK_CLEAR);
+  gtk_tooltips_set_tip (tooltips, widget, tip, NULL);
+  if (alloc)
+    free (tip);
+
+  g_signal_connect ((GObject *) widget, "clicked",
+		    G_CALLBACK (default_item_cb), (gpointer) entry->key);
+  return widget;
+}
+
+static void
+revert_item_cb (GtkWidget *w, gpointer key)
+{
+  xine_cfg_entry_t entry;
+  xine_config_lookup_entry (xine, key, &entry);
+  if (entry.type == XINE_CONFIG_TYPE_STRING)
+    entry.str_value = g_object_get_data ((GObject *) w, "revert");
+  else
+    entry.num_value = (int) (intptr_t) g_object_get_data ((GObject *) w, "revert");
+  preferences_update_entry (&entry);
+}
+
+static inline GtkWidget *
+create_item_revert (const xine_cfg_entry_t *entry)
+{
+  intptr_t value;
+  char *tip;
+  gboolean alloc = FALSE;
+
+  switch (entry->type)
+  {
+  case XINE_CONFIG_TYPE_ENUM:
+    tip = entry->enum_values[entry->num_value];
+    value = entry->num_value;
+    break;
+
+  case XINE_CONFIG_TYPE_RANGE:
+  case XINE_CONFIG_TYPE_NUM:
+    alloc = TRUE;
+    tip = g_strdup_printf ("%d", entry->num_value);
+    value = entry->num_value;
+    break;
+
+  case XINE_CONFIG_TYPE_BOOL:
+    tip = entry->num_value ? "✓" : "✗";
+    value = entry->num_value;
+    break;
+
+  case XINE_CONFIG_TYPE_STRING:
+    tip = entry->str_value;
+    value = (intptr_t) strdup (entry->str_value);
+    break;
+
+  default:
+    return NULL;
+  }
+
+  GtkWidget *widget = ui_button_new_stock (GTK_STOCK_REVERT_TO_SAVED);
+  g_object_set_data ((GObject *) widget, "revert", (void *) value);
+  g_signal_connect ((GObject *) widget, "clicked",
+		    G_CALLBACK (revert_item_cb), (gpointer) entry->key);
+  gtk_tooltips_set_tip (tooltips, widget, tip, NULL);
+  if (alloc)
+    free (tip);
+
+  return widget;
 }
 
 static GtkWidget *create_item_editable (const xine_cfg_entry_t *entry)
@@ -351,16 +592,17 @@ static GtkWidget *create_item_editable (const xine_cfg_entry_t *entry)
     break;
 
   case XINE_CONFIG_TYPE_STRING:
-    if (xine_check_version (1, 1, 4) && entry->num_value)
+    if (entry->num_value)
     {
-      widget = gtk_file_chooser_dialog_new (entry->key, GTK_WINDOW (prefs_dialog),
+      GtkWidget *fc = gtk_file_chooser_dialog_new (entry->key, GTK_WINDOW (prefs_dialog),
 					    entry->num_value == 3 /* XINE_CONFIG_STRING_IS_DIRECTORY_NAME */
 					      ? GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER
 					      : GTK_FILE_CHOOSER_ACTION_OPEN,
 					    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					    GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
 					    NULL);
-      gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (widget), TRUE);
+      widget = gtk_file_chooser_button_new_with_dialog (fc);
+      gtk_file_chooser_set_local_only ((GtkFileChooser *) fc, TRUE);
       struct stat st;
       if (stat (entry->str_value, &st))
       {
@@ -370,24 +612,33 @@ static GtkWidget *create_item_editable (const xine_cfg_entry_t *entry)
 		  entry->key, entry->str_value);
       }
       else
-	gtk_file_chooser_set_filename ((GtkFileChooser *) widget, entry->str_value);
-      g_object_connect (G_OBJECT(widget),
-	"signal::response", G_CALLBACK(file_cb), (gchar *)entry->key,
-	"signal::file-activated", G_CALLBACK(file_activate_cb), (gchar *)entry->key,
+	gtk_file_chooser_set_filename ((GtkFileChooser *) fc, entry->str_value);
+#ifdef WITH_HAL
+      gtk_file_chooser_set_use_preview_label ((GtkFileChooser *) fc, FALSE);
+      GtkWidget *preview = g_object_new (GTK_TYPE_LABEL, "angle", 90.0,
+					 "selectable", FALSE, NULL);
+      gtk_file_chooser_set_preview_widget ((GtkFileChooser *) fc, preview);
+#endif
+      g_object_connect (G_OBJECT(fc),
+	"signal::response", G_CALLBACK(file_cb), widget,
+	"signal::file-activated", G_CALLBACK(file_activate_cb), widget,
+#ifdef WITH_HAL
+	"signal::update-preview", G_CALLBACK(file_preview_cb), widget,
+	"signal::map", G_CALLBACK(file_preview_cb), fc,
+#endif
 	NULL);
-      widget = gtk_file_chooser_button_new_with_dialog (widget);
-      g_signal_connect (G_OBJECT(widget), "map", G_CALLBACK(file_map_cb), (gchar *)entry->key);
+      g_signal_connect_after (G_OBJECT(widget), "map", G_CALLBACK(file_map_cb), fc);
     }
     else
     {
       widget = gtk_entry_new();
-      g_object_set_data (G_OBJECT(widget), "cfg", (gpointer)entry->key);
       g_object_connect (G_OBJECT(widget),
 	"signal::key-press-event", G_CALLBACK(entry_keypress_cb), (gchar *)entry->key,
 	"signal::focus-out-event", G_CALLBACK(entry_cb), (gchar *)entry->key,
 	NULL);
       gtk_entry_set_text (GTK_ENTRY(widget), entry->str_value);
     }
+    g_object_set_data (G_OBJECT(widget), "cfg", (gpointer)entry->key);
 
     break;
 
@@ -395,10 +646,8 @@ static GtkWidget *create_item_editable (const xine_cfg_entry_t *entry)
     {
       GtkObject *adj = gtk_adjustment_new (entry->num_value, entry->range_min,
 					   entry->range_max, 1.0, 10.0, 0.0);
-      widget = gtk_hscale_new (GTK_ADJUSTMENT(adj));
-      gtk_scale_set_draw_value (GTK_SCALE(widget), TRUE);
-      gtk_scale_set_value_pos (GTK_SCALE(widget), GTK_POS_TOP);
-      gtk_scale_set_digits (GTK_SCALE(widget), 0);
+      widget = ui_hscale_new (adj, GTK_POS_TOP, 0);
+      gtk_widget_set_size_request (widget, 128, -1);
       g_signal_connect (adj, "value-changed",
 			G_CALLBACK (range_cb), (gchar *)entry->key);
     }
@@ -408,10 +657,7 @@ static GtkWidget *create_item_editable (const xine_cfg_entry_t *entry)
     {
       GtkObject *adj = gtk_adjustment_new (entry->num_value, INT_MIN, INT_MAX,
 					   1, 10, 0);
-      widget = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
-      gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON(widget),
-					 GTK_UPDATE_ALWAYS);
-      gtk_spin_button_set_numeric (GTK_SPIN_BUTTON(widget), TRUE);
+      widget = ui_spin_button_new (adj);
       g_signal_connect (G_OBJECT(widget), "value-changed",
 			G_CALLBACK(spin_cb), (gchar *)entry->key);
     }
@@ -449,23 +695,42 @@ static GtkWidget *create_item_label (const xine_cfg_entry_t *entry)
     labeltext = g_markup_printf_escaped ("<b>• <i>%s</i></b>\n%s",
 					 labelkey, entry->description);
 
-  label = gtk_label_new (labeltext);
+  label = ui_label_new_with_markup (labeltext);
   free (labeltext);
 
-  gtk_label_set_use_markup (GTK_LABEL(label), TRUE);
   gtk_label_set_line_wrap (GTK_LABEL(label), TRUE);
+  gtk_label_set_line_wrap_mode (GTK_LABEL(label), PANGO_WRAP_WORD);
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 
   return label;
 }
 
+static GtkWidget *create_item_extra (const xine_cfg_entry_t *entry)
+{
+  switch (entry->type)
+  {
+  case XINE_CONFIG_TYPE_STRING:
+#ifdef WITH_HAL
+    if (entry->num_value == 2 /* XINE_CONFIG_STRING_IS_DEVICE_NAME */)
+      return gtk_label_new (NULL);
+#endif
+    break;
+  }
+  return NULL;
+}
+
 static GNode *create_item (const xine_cfg_entry_t *entry)
 {
   pref_item_t *item = malloc (sizeof (pref_item_t));
   item->notebook = NULL;
+  item->todefault = create_item_default (entry);
+  item->revert = create_item_revert (entry);
   item->editable = create_item_editable (entry);
   item->label = create_item_label (entry);
+  GtkWidget *extra = create_item_extra (entry);
+  if (extra)
+    g_object_set_data (G_OBJECT (item->editable), "extra", extra);
   item->separator = gtk_hseparator_new ();
   item->exp = entry->exp_level;
   return g_node_new (item);
@@ -476,8 +741,7 @@ static GNode *create_page (const char *key, size_t length)
   pref_page_t *page = malloc (sizeof (pref_page_t));
   page->notebook = GINT_TO_POINTER (1); /* dummy value - filled in later */
   page->page = page->table = NULL;
-  page->prefix = malloc (length + 1);
-  sprintf (page->prefix, "%.*s", (int) length, key);
+  page->prefix = g_strdup_printf ("%.*s", (int) length, key);
   page->label = strrchr (page->prefix, '.');
   if (page->label)
     ++page->label;
@@ -634,14 +898,38 @@ static gboolean put_content (GNode *node, gpointer data)
     /* we have a config item */
     int r = g_node_child_position (node->parent, node) * 2;
     GtkTable *table = GTK_TABLE(parent->page.table);
-    gtk_table_attach (table, pref->item.editable, 0, 1, r, r + 1,
-		      GTK_EXPAND | GTK_FILL, GTK_SHRINK, 2, 5);
-    gtk_table_attach (table, pref->item.label, 1, 4, r, r + 1,
+    gtk_table_attach (table, pref->item.todefault, 0, 1, r, r + 1,
+		      GTK_FILL, GTK_SHRINK, 2, 5);
+    gtk_table_attach (table, pref->item.revert, 1, 2, r, r + 1,
+		      GTK_FILL, GTK_SHRINK, 2, 5);
+    GtkWidget *extra = g_object_get_data (G_OBJECT (pref->item.editable), "extra");
+    if (extra)
+    {
+      GtkBox *box = GTK_BOX (gtk_vbox_new (2, FALSE));
+      gtk_table_attach (table, (GtkWidget *) box, 2, 3, r, r + 1,
+			GTK_EXPAND | GTK_FILL, GTK_SHRINK, 2, 5);
+      gtk_box_pack_start_defaults (box, pref->item.editable);
+      gtk_box_pack_start_defaults (box, extra);
+#ifdef WITH_HAL
+      xine_cfg_entry_t entry;
+      const char *key = g_object_get_data (G_OBJECT (pref->item.editable), "cfg");
+      if (xine_config_lookup_entry (xine, key, &entry))
+	display_hal_device_info (pref->item.editable, entry.str_value);
+#endif
+    }
+    else
+      gtk_table_attach (table, pref->item.editable, 2, 3, r, r + 1,
+			GTK_EXPAND | GTK_FILL, GTK_SHRINK, 2, 5);
+    gtk_table_attach (table, pref->item.label, 3, 6, r, r + 1,
 		      GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
-    gtk_table_attach (table, pref->item.separator, 0, 4, r + 1, r + 2,
+    gtk_table_attach (table, pref->item.separator, 0, 6, r + 1, r + 2,
 		      GTK_EXPAND | GTK_FILL, GTK_SHRINK | GTK_FILL, 5, 5);
+    g_signal_connect (pref->item.todefault, "focus",
+		      G_CALLBACK (focus_item_cb), node);
+    g_signal_connect (pref->item.revert, "focus",
+		      G_CALLBACK (focus_item_cb), node);
     g_signal_connect (pref->item.editable, "focus",
-		      focus_item_cb, node);
+		      G_CALLBACK (focus_item_cb), node);
   }
 
   return FALSE;
@@ -729,7 +1017,7 @@ static JSBool js_preferences_show (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
-static gpointer preferences_init_dbox (gpointer unused)
+static void preferences_init_dbox (void)
 {
   prefs_dialog = gtk_dialog_new_with_buttons (_("Preferences"), NULL, 0,
 				GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
@@ -745,22 +1033,23 @@ static gpointer preferences_init_dbox (gpointer unused)
   gtk_box_pack_start_defaults (GTK_BOX(GTK_DIALOG(prefs_dialog)->vbox),
 			       prefs_notebook);
 
-  GtkWidget *w = gtk_label_new (_("Items marked “<b>• <i>like this</i></b>” require "
-				  "gxine to be restarted to take effect."));
-  gtk_label_set_use_markup (GTK_LABEL(w), TRUE);
+  GtkWidget *w = ui_label_new_with_markup
+		   (_("Items marked “<b>• <i>like this</i></b>” require "
+		      "gxine to be restarted to take effect."));
   gtk_box_pack_end (GTK_BOX(GTK_DIALOG(prefs_dialog)->vbox), w,
 		    FALSE, FALSE, 2);
+}
 
-  gtk_widget_show_all (prefs_notebook);
-
+static gpointer preferences_first_show (gpointer unused)
+{
+  static int value = 0;
   xine_cfg_entry_t entry;
-  if (xine_config_lookup_entry (xine, "gui.experience_level", &entry))
-  {
-    static int value;
-    value = entry.num_value;
-    select_show_pref_widgets (&value);
-  }
 
+  gtk_widget_show_all (GTK_DIALOG (prefs_dialog)->vbox);
+  if (xine_config_lookup_entry (xine, "gui.experience_level", &entry))
+    value = entry.num_value;
+
+  select_show_pref_widgets (&value);
   return NULL;
 }
 
@@ -775,8 +1064,8 @@ void preferences_show (void)
   {
     is_visible = TRUE;
     static GOnce once = G_ONCE_INIT;
-    g_once (&once, preferences_init_dbox, NULL);
-    window_show (prefs_dialog, NULL);
+    g_once (&once, preferences_first_show, NULL);
+    window_present (prefs_dialog, NULL);
   }
 }
 
@@ -786,6 +1075,8 @@ void preferences_init (void)
   /* script engine functions */
   se_defun (gse, NULL, "preferences_show", js_preferences_show, 0, 0,
 	    SE_GROUP_DIALOGUE, NULL, NULL);
+
+  preferences_init_dbox ();
 }
 
 void preferences_update_entry (const xine_cfg_entry_t *entry)
@@ -800,8 +1091,13 @@ void preferences_update_entry (const xine_cfg_entry_t *entry)
       break;
 
     case XINE_CONFIG_TYPE_STRING:
-      if (xine_check_version (1, 1, 4) && entry->num_value)
+      if (entry->num_value)
+      {
 	gtk_file_chooser_set_filename (widget, entry->str_value);
+#ifdef WITH_HAL
+	display_hal_device_info (widget, entry->str_value);
+#endif
+      }
       else
 	gtk_entry_set_text (widget, entry->str_value);
       break;
