@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2006 the xine project
+ * Copyright (C) 2003-2007 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,8 +15,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: script_engine.c,v 1.84 2006/04/08 21:34:50 dsalt Exp $
- *
  * gate to the spider monkey javascript interpreter
  *
  * provides functions to generate interpreter instances
@@ -49,6 +47,7 @@
 #include "log_window.h"
 #include "stream_info.h"
 #include "open_mrl.h"
+#include "player.h"
 #include "server.h"
 #include "wizards.h"
 #include "snapshot.h"
@@ -261,7 +260,7 @@ static JSBool controls_exit (JSContext *cx, JSObject *obj, uintN argc,
 
   se_log_fncall_checkinit ("controls.exit");
 
-  xine_stop (stream);
+  player_stop ();
 
   if (gtk_main_level() < 1)
   {
@@ -295,6 +294,7 @@ static JSBool controls_exit (JSContext *cx, JSObject *obj, uintN argc,
   settings_clear ();
 
   gtk_main_quit();
+  sched_yield ();
 
   exit (0);
 }
@@ -652,7 +652,7 @@ static jsval get_prop_jsval (se_t *se, se_o_t *o, se_prop_t *p)
 
   case SE_TYPE_DOUBLE:
     {
-      double d = get_prop_int (se, o, p);
+      jsdouble *d = JS_NewDouble (se->cx, get_prop_double (se, o, p));
       return DOUBLE_TO_JSVAL (d);
     }
 
@@ -972,7 +972,7 @@ void se_prop_set (se_t *se, se_o_t *o, const char *id, const char *value)
     se_prop_listeners (se, o, p, v);
   }
   else
-    se_prop_create (se, o, id, value, NULL, 0, SE_TYPE_STRING, FALSE);
+    se_prop_create (se, o, id, value, SE_TYPE_STRING, 0, FALSE);
 }
 
 void se_prop_set_int (se_t *se, se_o_t *o, const char *id, int value)
@@ -1195,8 +1195,7 @@ JSBool se_prop_get_bool (se_t *se, se_o_t *o, const char *id)
 
 void se_prop_create (se_t *se, se_o_t *o,
                      const char *id, const char *value,
-		     const char *xine_id, int xine_param,
-		     se_prop_type_t se_type, gboolean constant)
+		     se_prop_type_t se_type, intptr_t data, gboolean constant)
 {
   if (!o)
     o = se->g;
@@ -1210,10 +1209,9 @@ void se_prop_create (se_t *se, se_o_t *o,
     p = malloc (sizeof (se_prop_t));
     p->id = strdup (id);
     p->value = strdup (value);
-    p->xine_id = xine_id ? strdup (xine_id) : NULL;
-    p->xine_param = xine_param;
     p->reader = NULL;
     p->listeners = NULL;
+    p->data = data;
     o->properties = g_list_append (o->properties, p);
   }
 
@@ -1221,12 +1219,20 @@ void se_prop_create (se_t *se, se_o_t *o,
   p->constant = constant;
 }
 
+void se_prop_create_with_reader (se_t *se, se_o_t *o,
+				 const char *id, const char *value,
+				 se_prop_type_t se_type, intptr_t data,
+				 gboolean constant, se_prop_reader_t reader)
+{ se_prop_create (se, o, id, value, se_type, data, constant);
+  se_prop_set_reader_func (se, o, id, reader);
+}
+
 void se_prop_create_int (se_t *se, se_o_t *o, const char *id, int value,
 			 gboolean constant)
 {
   char str[24];
   snprintf (str, sizeof (str), "%d", value);
-  se_prop_create (se, o, id, str, NULL, 0, SE_TYPE_INT, constant);
+  se_prop_create (se, o, id, str, SE_TYPE_INT, 0, constant);
 }
 
 void se_prop_create_double (se_t *se, se_o_t *o, const char *id, double value,
@@ -1234,7 +1240,7 @@ void se_prop_create_double (se_t *se, se_o_t *o, const char *id, double value,
 {
   char str[24];
   snprintf (str, sizeof (str), "%la", value); /* C99 hex float */
-  se_prop_create (se, o, id, str, NULL, 0, SE_TYPE_DOUBLE, constant);
+  se_prop_create (se, o, id, str, SE_TYPE_DOUBLE, 0, constant);
 }
 
 void se_prop_create_bool (se_t *se, se_o_t *o, const char *id, int value,
@@ -1242,22 +1248,22 @@ void se_prop_create_bool (se_t *se, se_o_t *o, const char *id, int value,
 {
   char str[24];
   snprintf (str, sizeof (str), "%d", !!value);
-  se_prop_create (se, o, id, str, NULL, 0, SE_TYPE_BOOL, constant);
+  se_prop_create (se, o, id, str, SE_TYPE_BOOL, 0, constant);
 }
 
-static void se_prop_read_xine_id_int (se_prop_t *prop, se_prop_read_t *value)
+static void se_prop_read_xine_id_int (const se_prop_t *prop, se_prop_read_t *value)
 {
   xine_cfg_entry_t entry;
-  if (xine_config_lookup_entry (xine, prop->xine_id, &entry))
+  if (xine_config_lookup_entry (xine, (const char *) prop->data, &entry))
     value->i = entry.num_value;
   else
     value->i = 0;
 }
 
-static void se_prop_read_xine_id_string (se_prop_t *prop, se_prop_read_t *value)
+static void se_prop_read_xine_id_string (const se_prop_t *prop, se_prop_read_t *value)
 {
   xine_cfg_entry_t entry;
-  if (xine_config_lookup_entry (xine, prop->xine_id, &entry))
+  if (xine_config_lookup_entry (xine, (const char *) prop->data, &entry))
     value->s = entry.str_value;
   else
     value->s = "";
@@ -1272,28 +1278,31 @@ void se_prop_create_xine_id (se_t *se, se_o_t *o,
   {
   case XINE_CONFIG_TYPE_RANGE:
   case XINE_CONFIG_TYPE_NUM:
-    se_prop_create (se, o, id, ".", xine_id, 0, SE_TYPE_INT, FALSE);
-    se_prop_set_reader_func (se, o, id, se_prop_read_xine_id_int);
-    return;
+    se_prop_create_with_reader (se, o, id, ".", SE_TYPE_INT,
+				(intptr_t) xine_id, FALSE,
+				se_prop_read_xine_id_int);
+    break;
   case XINE_CONFIG_TYPE_ENUM:
     g_printerr (_("script_engine: can't create property for xine config enum type\n"));
-    return;
+    break;
   case XINE_CONFIG_TYPE_STRING:
-    se_prop_create (se, o, id, "", xine_id, 0, SE_TYPE_STRING, FALSE);
-    se_prop_set_reader_func (se, o, id, se_prop_read_xine_id_string);
-    return;
+    se_prop_create_with_reader (se, o, id, ".", SE_TYPE_STRING,
+				(intptr_t) xine_id, FALSE,
+				se_prop_read_xine_id_string);
+    break;
   case XINE_CONFIG_TYPE_BOOL:
-    se_prop_create (se, o, id, "", xine_id, 0, SE_TYPE_BOOL, FALSE);
-    se_prop_set_reader_func (se, o, id, se_prop_read_xine_id_int);
-    return;
+    se_prop_create_with_reader (se, o, id, ".", SE_TYPE_BOOL,
+				(intptr_t) xine_id, FALSE,
+				se_prop_read_xine_id_int);
+    break;
   default:
     g_printerr (_("script_engine: can't create property for xine config unknown type\n"));
   }
 }
 
-static void se_prop_read_xine_param (se_prop_t *prop, se_prop_read_t *value)
+static void se_prop_read_xine_param (const se_prop_t *prop, se_prop_read_t *value)
 {
-  value->i = xine_get_param (stream, prop->xine_param);
+  value->i = xine_get_param (stream, prop->data);
 }
 
 void se_prop_create_xine_param (se_t *se, se_o_t *o,
@@ -1304,8 +1313,8 @@ void se_prop_create_xine_param (se_t *se, se_o_t *o,
   {
   case SE_TYPE_BOOL:
   case SE_TYPE_INT:
-    se_prop_create (se, o, id, "", NULL, xine_param, type, FALSE);
-    se_prop_set_reader_func (se, o, id, se_prop_read_xine_param);
+    se_prop_create_with_reader (se, o, id, "", type, xine_param, FALSE,
+				se_prop_read_xine_param);
     return;
   default:
     g_printerr (_("script_engine: can't create property for xine param non-int type\n"));
@@ -1590,7 +1599,7 @@ se_t *se_new (void)
       { "exit", controls_exit, 0, 0, SE_GROUP_ENGINE, NULL, NULL },
       { "js_console_show", show_js_console, 0, 0, SE_GROUP_DIALOGUE, NULL, NULL },
       { "help", show_help, 0, 0, SE_GROUP_HIDDEN, NULL, NULL },
-      { "callback", js_callback, 0, 0, SE_GROUP_ENGINE, NULL, NULL },
+      { "callback", js_callback, 0, 0, SE_GROUP_ENGINE, N_("Javascript"), NULL },
       { "xine_cfg_get", js_xine_cfg_get, 0, 0, SE_GROUP_PROPERTIES, NULL, NULL },
       { "xine_cfg_set", js_xine_cfg_set, 0, 0, SE_GROUP_PROPERTIES, NULL, NULL },
       { NULL }
@@ -1600,8 +1609,7 @@ se_t *se_new (void)
 
   event_obj = se_create_object (se, NULL, "event", NULL, SE_GROUP_ENGINE,
 				"stream_end=JS");
-  se_prop_create (se, event_obj, "stream_end", "", NULL, 0,
-		  SE_TYPE_STRING, FALSE);
+  se_prop_create (se, event_obj, "stream_end", "", SE_TYPE_STRING, 0, FALSE);
 
   js_icon_obj = se_create_object (se, NULL, "__js", NULL, SE_GROUP_HIDDEN, NULL);
   se_prop_create_int (se, js_icon_obj, "v", 0, TRUE);
