@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2005 the xine project
+ * Copyright (C) 2003-2007 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,8 +15,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: snapshot.c,v 1.20 2006/03/27 22:11:25 dsalt Exp $
- *
  * snapshot implementation
  *
  * code based on:
@@ -145,7 +143,8 @@ void make_snapshot (char *fname, int scale, int blend)
 
   /* catch "no image" and unrecognised formats */
 
-  switch (format) {
+  switch (format)
+  {
   case XINE_IMGFMT_YUY2:
   case XINE_IMGFMT_YV12:
     break;
@@ -189,7 +188,8 @@ void make_snapshot (char *fname, int scale, int blend)
 
   /* convert to rgb */
 
-  switch (format) {
+  switch (format)
+  {
   case XINE_IMGFMT_YUY2:
     rgb = yuy2torgb (yuv, width, height);
     break;
@@ -236,7 +236,8 @@ void make_snapshot (char *fname, int scale, int blend)
   else
     image_ratio = (double) width / (double) height;
 
-  switch (ratio) {
+  switch (ratio)
+  {
   case XINE_VO_ASPECT_ANAMORPHIC:  /* anamorphic     */
   case XINE_VO_ASPECT_PAN_SCAN:    /* we display pan&scan as widescreen */
     desired_ratio = 16.0 /9.0;
@@ -266,10 +267,10 @@ void make_snapshot (char *fname, int scale, int blend)
     scaled = gdk_pixbuf_scale_simple (pixbuf,
 				      width, (int) ((double) height / f),
 				      (GdkInterpType) blend);
-    
+
   gdk_pixbuf_unref (pixbuf);
-    
-  /* 
+
+  /*
    * save pixbuf to file
    */
 
@@ -284,23 +285,25 @@ void snapshot_init (void)
 {
   int i;
   GtkWidget *w;
+  GtkBox *b;
 
   bbox = gtk_hbox_new (FALSE, 2);
+  b = GTK_BOX (bbox);
   gtk_widget_ref (bbox);
 
   w = gtk_label_new_with_mnemonic (_("_Blend:"));
-  gtk_box_pack_start (GTK_BOX(bbox), w, FALSE, FALSE, 2);
+  gtk_box_pack_start (b, w, FALSE, FALSE, 2);
 
   blendbtn = gtk_combo_box_new_text ();
   for (i = 0; blends[i]; ++i)
     gtk_combo_box_append_text (GTK_COMBO_BOX(blendbtn), gettext (blends[i]));
-  gtk_box_pack_start (GTK_BOX(bbox), blendbtn, FALSE, FALSE, 2);
+  gtk_box_pack_start (b, blendbtn, FALSE, FALSE, 2);
   gtk_combo_box_set_active (GTK_COMBO_BOX(blendbtn), 2); /* bilinear */
 
   gtk_label_set_mnemonic_widget (GTK_LABEL(w), blendbtn);
 
   scalebtn = gtk_check_button_new_with_mnemonic (_("_Scaled"));
-  gtk_box_pack_start (GTK_BOX(bbox), scalebtn, FALSE, FALSE, 2);
+  gtk_box_pack_start (b, scalebtn, FALSE, FALSE, 2);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(scalebtn), TRUE);
 
   gtk_widget_show_all (bbox);
