@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2005 the xine-project
+ * Copyright (C) 2001-2007 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,13 +16,11 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: ui.h,v 1.19 2006/04/07 18:06:40 dsalt Exp $
- *
  * needful things
  */
 
-#ifndef HAVE_UI_H
-#define HAVE_UI_H
+#ifndef GXINE_UI_H
+#define GXINE_UI_H
 
 #include <gtk/gtk.h>
 #include <xine.h>
@@ -76,6 +74,7 @@ typedef enum {
   Control_COMPRESSOR,
   Control_AMPLIFIER,
   Control_AV_SYNC,
+  Control_SPU_SYNC,
   Control_HUE,
   Control_SATURATION,
   Control_CONTRAST,
@@ -117,14 +116,52 @@ void ui_clear_control_adjustment (control_adjustment_t);
 /* Update objects which use the given adjustment */
 void ui_xine_set_param_from_adjustment (control_adjustment_t);
 
+void ui_register_adjustment_widget (control_adjustment_t, GtkWidget *);
+
 void ui_preferences_register (xine_t *);
 
 /* Property creation */
-typedef struct {
+enum ui_property_type_t {
+  UI_PROP_BOOL,
+  UI_PROP_XINE_PARAM,
+  UI_PROP_GTK_WIDGET,
+  UI_PROP_GTK_VIDEO,
+  UI_PROP_CUSTOM,
+};
+#define UI_BOOL(ACTION, VAR, SET) { .flag = { SET, &VAR, &action_items.ACTION } }, UI_PROP_BOOL
+#define UI_XINE_PARAM(PARAM, SET) { .xine = { SET, PARAM } }, UI_PROP_XINE_PARAM
+#define UI_GTK_WIDGET(WIDGET, GET, SET) { .widget = { SET, GET, &WIDGET } }, UI_PROP_GTK_WIDGET
+#define UI_GTK_VIDEO(ACTION, GET) { .video = { NULL, GET, &action_items.ACTION } }, UI_PROP_GTK_VIDEO
+#define UI_CUSTOM(GET, SET) { .custom = { SET, GET } }, UI_PROP_CUSTOM
+
+typedef struct ui_property_s {
   const char *name, *help;
-  int param;
-  se_prop_reader_t get;
-  void (*set) (int);
+  union {
+    struct {
+      void (*set) (int);
+      gboolean *flag;
+      GtkToggleAction **action;
+    } flag;
+    struct {
+      void (*set) (int);
+      int param;
+    } xine;
+    struct {
+      void (*set) (GtkWidget *, gint);
+      int (*get) (GtkWidget *);
+      GtkWidget **widget;
+    } widget;
+    struct {
+      void (*set) (); /* unused */
+      int (*get) (gpointer); //(GtkVideo *);
+      GtkToggleAction **action;
+    } video;
+    struct {
+      void (*set) (int);
+      int (*get) (const struct ui_property_s *, se_prop_read_t *);
+    } custom;
+  } ui;
+  enum ui_property_type_t type;
   int min, max;
   gboolean loop;
   se_prop_listener_t listen;
@@ -144,6 +181,12 @@ GtkWidget *ui_toggle_button_new_icon_name (const char *);
 GtkWidget *ui_button_new_icon_name_mnemonic (const char *name,
 					     const char *mnemonic);
 
+/* Misc creation */
+GtkWidget *ui_spin_button_new (GtkObject *adj);
+GtkWidget *ui_hscale_new (GtkObject *adj, GtkPositionType, int digits);
+GtkWidget *ui_label_new_with_xalign (const char *, gfloat);
+GtkWidget *ui_label_new_with_markup (const char *);
+
 /* Menu/toolbar basics */
 GtkUIManager *ui_create_manager (const char *label, GtkWidget *window);
 #define ui_get_action_group(ui) \
@@ -164,6 +207,7 @@ void window_wm_toolbar_restore (void);
 void window_wm_toolbar_set_snap (gboolean);
 gboolean window_wm_toolbar_snap (void);
 void window_check_vis (gboolean force);
+void window_fs_stickiness_update (gboolean);
 
 void ui_init (void);
 gboolean ui_post_init (void);
