@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2006 the xine-project
+ * Copyright (C) 2001-2007 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,8 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: wizards.c,v 1.46 2006-03-08 23:42:21 dsalt Exp $
- *
  * stuff to run on first startup / setup wizards
  */
 
@@ -44,17 +42,16 @@
 #include "desktop_integration.h"
 #include "preferences.h"
 
-static GtkWidget *add_label (GtkWidget *vbox, const char *label)
+static GtkWidget *add_label (GtkBox *vbox, const char *label)
 {
-  GtkWidget *l = gtk_label_new (label);
+  GtkWidget *l = ui_label_new_with_markup (label);
   gtk_label_set_line_wrap (GTK_LABEL (l), TRUE);
-  gtk_label_set_use_markup (GTK_LABEL (l), TRUE);
   gtk_misc_set_alignment (GTK_MISC (l), 0, 0.5);
-  gtk_box_pack_start (GTK_BOX(vbox), l, FALSE, FALSE, 5);
+  gtk_box_pack_start (vbox, l, FALSE, FALSE, 5);
   return l;
 }
 
-static GtkWidget *add_heading (GtkWidget *vbox, const char *label)
+static GtkWidget *add_heading (GtkBox *vbox, const char *label)
 {
   char *markup = g_strdup_printf ("<big><b>%s</b></big>", label);
   GtkWidget *l = add_label (vbox, markup);
@@ -62,7 +59,7 @@ static GtkWidget *add_heading (GtkWidget *vbox, const char *label)
   return l;
 }
 
-static gboolean do_welcome (GtkWidget *dlg, GtkWidget *vbox)
+static gboolean do_welcome (GtkDialog *dlg, GtkBox *vbox)
 {
   add_heading (vbox, _("Welcome to gxine!\n\n"));
   add_label (vbox, _("Would you like to run some setup wizards now "
@@ -96,7 +93,7 @@ static void hc_update_pref (const char *pref, const char *value)
   preferences_update_entry (&entry);
 }
 
-static gboolean do_health_check (GtkWidget *dlg, GtkWidget *vbox)
+static gboolean do_health_check (GtkDialog *dlg, GtkBox *vbox)
 {
   GtkWidget           *l, *table, *b;
   xine_health_check_t  hc;
@@ -155,10 +152,9 @@ static gboolean do_health_check (GtkWidget *dlg, GtkWidget *vbox)
 
     gtk_table_resize (GTK_TABLE (table), 3, check+1);
 
-    l = gtk_label_new (results->title);
+    l = ui_label_new_with_xalign (results->title, 0);
     gtk_table_attach (GTK_TABLE (table), l, 0, 1, check, check+1, GTK_FILL,
 		      GTK_SHRINK, 2, 5);
-    gtk_misc_set_alignment (GTK_MISC (l), 0, 0.5);
 
     switch (results->status)
     {
@@ -190,59 +186,57 @@ static gboolean do_health_check (GtkWidget *dlg, GtkWidget *vbox)
 }
 
 #ifdef USE_INTEGRATION_WIZARD
-static GtkWidget *cb_moz;
-static GtkWidget *cb_gnome;
-static GtkWidget *cb_kde;
-static GtkWidget *cb_mailcap;
+static struct {
+  GtkToggleButton *moz, *gnome, *kde, *mailcap;
+} cb;
 
 static void integrate_cb (GtkButton *button, gpointer data)
 {
   gboolean state = !!data;
-  if (GTK_WIDGET_SENSITIVE (cb_mailcap))
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_mailcap), state);
-  if (GTK_WIDGET_SENSITIVE (cb_moz))
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_moz), state);
-  if (GTK_WIDGET_SENSITIVE (cb_gnome))
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_gnome), state);
-  if (GTK_WIDGET_SENSITIVE (cb_kde))
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_kde), state);
+  if (GTK_WIDGET_SENSITIVE (cb.mailcap))
+    gtk_toggle_button_set_active (cb.mailcap, state);
+  if (GTK_WIDGET_SENSITIVE (cb.moz))
+    gtk_toggle_button_set_active (cb.moz, state);
+  if (GTK_WIDGET_SENSITIVE (cb.gnome))
+    gtk_toggle_button_set_active (cb.gnome, state);
+  if (GTK_WIDGET_SENSITIVE (cb.kde))
+    gtk_toggle_button_set_active (cb.kde, state);
 }
 
-static gboolean ask_integration_wizard (GtkWidget *w, GtkWidget *vbox)
+static GtkToggleButton *
+new_toggle (GtkBox *box, const char *label, gboolean active)
+{
+  GtkWidget *w = gtk_check_button_new_with_mnemonic (label);
+  gtk_widget_set_sensitive (w, active);
+  gtk_box_pack_start (box, w, FALSE, FALSE, 5);
+  GtkToggleButton *b = GTK_TOGGLE_BUTTON (w);
+  gtk_toggle_button_set_active (b, active);
+  return b;
+}
+
+static gboolean ask_integration_wizard (GtkDialog *w, GtkBox *vbox)
 {
   struct stat st;
   gchar *sname;
   GtkWidget *b, *hbox;
-  gboolean enable;
+  GtkBox *vb = GTK_BOX (vbox);
 
   add_heading (vbox, _("Registration"));
   add_label (vbox, _("Register gxine with the following applications "
 		     "as a media handler/helper:"));
 
-  cb_mailcap = gtk_check_button_new_with_mnemonic ("~_/.mailcap");
-  gtk_box_pack_start (GTK_BOX(vbox), cb_mailcap, FALSE, FALSE, 5);
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_mailcap), TRUE);
+  cb.mailcap = new_toggle (vb, "~_/.mailcap", TRUE);
 
-  cb_moz = gtk_check_button_new_with_mnemonic (_("_Mozilla & Mozilla Firefox (plugin)"));
-  gtk_box_pack_start (GTK_BOX(vbox), cb_moz, FALSE, FALSE, 5);
   sname = g_build_filename (plugindir, "gxineplugin.so", NULL);
-  enable = !stat (sname, &st);
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_moz), enable);
-  gtk_widget_set_sensitive (cb_moz, enable);
+  cb.moz = new_toggle (vb, _("_Mozilla & Mozilla Firefox (plugin)"), !stat (sname, &st));
   g_free (sname);
 
-  cb_gnome = gtk_check_button_new_with_mnemonic (_("_GNOME, Nautilus"));
-  gtk_box_pack_start (GTK_BOX(vbox), cb_gnome, FALSE, FALSE, 5);
-  enable = di_gnome_vfs_init ();
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_gnome), enable);
-  gtk_widget_set_sensitive (cb_gnome, enable);
+  cb.gnome = new_toggle (vb, _("_GNOME, Nautilus"), gxine_vfs_init ());
 
-  cb_kde = gtk_check_button_new_with_mnemonic (_("_KDE, Konqueror"));
-  gtk_box_pack_start (GTK_BOX(vbox), cb_kde, FALSE, FALSE, 5);
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_kde), TRUE);
+  cb.kde = new_toggle (vb, _("_KDE, Konqueror"), TRUE);
 
   hbox = gtk_hbox_new (FALSE, 2);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 2);
+  gtk_box_pack_start (vb, hbox, FALSE, FALSE, 2);
 
   b = ui_button_new_stock_mnemonic (GTK_STOCK_ADD, _("_All"));
   gtk_box_pack_start (GTK_BOX (hbox), b, FALSE, FALSE, 2);
@@ -260,17 +254,17 @@ static gboolean ask_integration_wizard (GtkWidget *w, GtkWidget *vbox)
 static void desktop_integration (void)
 {
   di_registration_flush ();
-  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_mailcap)))
+  if (gtk_toggle_button_get_active (cb.mailcap))
     di_register_mailcap();
-  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_gnome)))
+  if (gtk_toggle_button_get_active (cb.gnome))
     di_register_gnome();
-  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_kde)))
+  if (gtk_toggle_button_get_active (cb.kde))
     di_register_kde();
-  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_moz)))
+  if (gtk_toggle_button_get_active (cb.moz))
     di_register_mozilla();
 }
 
-static gboolean report_integration_wizard (GtkWidget *w, GtkWidget *vbox)
+static gboolean report_integration_wizard (GtkDialog *w, GtkBox *vbox)
 {
   const char *report = di_registration_report ();
   if (!report)
@@ -315,7 +309,7 @@ static void response_cb (GtkDialog *dbox, int response, gpointer data)
 
 void run_wizards (gboolean requested)
 {
-  static GtkWidget *dlg = NULL;
+  static GtkDialog *dlg = NULL;
 
   if (dlg)
   {
@@ -327,13 +321,13 @@ void run_wizards (gboolean requested)
   if (requested
       || (xine_config_register_num (xine, "misc.wizards_shown", 0,
 				    _("Keep track of whether user has seen wizards yet"),
-				    NULL, 20, NULL, NULL) < WIZARDS_LEVEL))
+				    NULL, 32767, NULL, NULL) < WIZARDS_LEVEL))
   {
     xine_cfg_entry_t  entry;
     int b, state;
 
     static const struct {
-      GtkWidget *(*create) (GtkWidget *, GtkWidget *);
+      GtkWidget *(*create) (GtkDialog *, GtkBox *);
       gboolean (*action) (void); /* TRUE to skip a stage */
     } wizards[] = {
       { do_welcome, NULL },
@@ -353,11 +347,11 @@ void run_wizards (gboolean requested)
 
     /* set up dialog which all wizards will use */
 
-    dlg = gtk_dialog_new_with_buttons (_("gxine setup wizards"), NULL, 0,
+    dlg = GTK_DIALOG (gtk_dialog_new_with_buttons (_("gxine setup wizards"), NULL, 0,
 				       GTK_STOCK_GO_FORWARD, GTK_RESPONSE_ACCEPT,
 				       GTK_STOCK_CLOSE, GTK_RESPONSE_DELETE_EVENT,
-				       NULL);
-    gtk_dialog_set_default_response (GTK_DIALOG(dlg), GTK_RESPONSE_ACCEPT);
+				       NULL));
+    gtk_dialog_set_default_response (dlg, GTK_RESPONSE_ACCEPT);
     gtk_window_set_default_size (GTK_WINDOW (dlg), -1, 400);
     gtk_window_set_modal (GTK_WINDOW (dlg), !requested);
     g_object_connect (G_OBJECT (dlg),
@@ -372,35 +366,34 @@ void run_wizards (gboolean requested)
     /* headline */
     {
       GtkWidget *w, *b = gtk_hbox_new (0, 0);
-      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), b, FALSE, FALSE, 2);
+      gtk_box_pack_start (GTK_BOX (dlg->vbox), b, FALSE, FALSE, 2);
       gtk_box_pack_start (GTK_BOX (b),
 			  gtk_image_new_from_stock (GXINE_LOGO, icon_size_logo),
 			  FALSE, FALSE, 2);
-      w = gtk_label_new (_("<span face='serif' size='xx-large' weight='bold'>Welcome!</span>"));
-      gtk_label_set_use_markup (GTK_LABEL (w), TRUE);
+      w = ui_label_new_with_markup (_("<span face='serif' size='xx-large' weight='bold'>Welcome!</span>"));
       gtk_box_pack_start (GTK_BOX (b), w, FALSE, FALSE, 2);
     }
 
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox),
+    gtk_box_pack_start (GTK_BOX (dlg->vbox),
 			gtk_hseparator_new (), FALSE, FALSE, 5);
 
-    window_show (dlg, NULL);
+    window_show ((GtkWidget *)dlg, NULL);
 
     state = -1;
     while (1)
     {
       GtkWidget *content = gtk_vbox_new (0, 0);
-      gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dlg)->vbox), content);
+      gtk_box_pack_start_defaults (GTK_BOX (dlg->vbox), content);
 
       if (wizards[++state].create)
       {
-        if (wizards[state].create (dlg, content))
+        if (wizards[state].create (dlg, GTK_BOX (content)))
 	  goto next;
       }
       else
       {
-	gtk_dialog_set_response_sensitive (GTK_DIALOG (dlg), GTK_RESPONSE_ACCEPT, FALSE);
-	add_heading (content, _("Setup completed."));
+	gtk_dialog_set_response_sensitive (dlg, GTK_RESPONSE_ACCEPT, FALSE);
+	add_heading (GTK_BOX (content), _("Setup completed."));
       }
 
       gtk_widget_show_all (content);
@@ -413,10 +406,10 @@ void run_wizards (gboolean requested)
         wizards[state].action ();
 
       next:
-      gtk_container_remove (GTK_CONTAINER (GTK_DIALOG (dlg)->vbox), content);
+      gtk_container_remove (GTK_CONTAINER (dlg->vbox), content);
     }
 
-    gtk_widget_destroy (dlg);
+    gtk_widget_destroy (GTK_WIDGET (dlg));
     dlg = NULL;
   }
 }
