@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2004 the xine-project
+ * Copyright (C) 2001-2006 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,13 +16,11 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: wizards.h,v 1.3 2006-02-21 19:06:12 dsalt Exp $
- *
  * stuff to run on first startup / setup wizards
  */
 
-#ifndef HAVE_WIZARDS_H
-#define HAVE_WIZARDS_H
+#ifndef GXINE_WIZARDS_H
+#define GXINE_WIZARDS_H
 
 #include <glib.h>
 
